/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
 
netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");

	
function XMLTemplate(id, dom) {
	this.id = id;
	this.xmlTemplate = document.getElementById(this.id);
						
	this.XPATH = Components.classes["@mozilla.org/dom/xpath-evaluator;1"].createInstance(Components.interfaces.nsIDOMXPathEvaluator);
	this.NSResolver = this.XPATH.createNSResolver(this.xmlTemplate);
	this.ORDERED_NODE_ITERATOR_TYPE = 5; // Ordered.
	// load xml document.
	if (dom)
		this.dom = dom;
	else	{	
		this.uri	= this.xmlTemplate.getAttribute("text/xml", "uri")
		if (this.uri) {
			this.dom = Utility.loadDOMFromURI(this.uri);
			assert(this.dom, "Loading XML for " + this.uri + " failed.");
		}	
	}
	// store xmlparam's as associative array.
	this.buildParams();

	// load xmlpath
	this.xmlapply = this.xmlTemplate.getElementsByTagName("xmlapply").item(0);
	assert(this.xmlapply, "xmlapply tag not found for xmltemplate " + this.id + ".");
	
	// store rules as associative array.
	this.buildRules();
	this.xmlbody = this.xmlTemplate.getElementsByTagName("xmlbody").item(0);
	assert(this.xmlbody, "xmlbody tag not found for xmltemplate " + this.id + ".");
	
	// We need to pass on the orientation attribute to generated content. (others?)
	var orient = this.xmlTemplate.parentNode.getAttribute("orient");
	if (!orient)
		orient = "vertical";
	this.xmlbody.setAttribute("orient", orient);
	this.attributeFinderRE = new RegExp("(\?[^ ?!]+!?)");
	this.attributeExtractorRE = new RegExp("@([^!]+)!?");
}
XMLTemplate.prototype.build = function() {
	// if parameters are present we need to build the element list every time.
	if (this.paramsPresent) 
		this.buildParams(); // they might have changed.
	
	// Clear existing content
	while (this.xmlbody.firstChild) {
		this.xmlbody.removeChild(this.xmlbody.lastChild);
	}
	
	this.buildFromApply(this.dom, this.xmlapply, this.xmlbody, false);
}
XMLTemplate.prototype.buildFromApply = function(xmlnode, applyElement, insertPoint, insertAfter) {
	var currentInsertPoint = insertPoint;
	var elementList = this.buildElementList(xmlnode, applyElement);
	var node;
	while ((node = elementList.iterateNext())) {
		var xmlrule = this.xmlrules[node.localName];
		if (!xmlrule) 
			continue;
		var ruleElement = xmlrule.element	
		var ruleContent = ruleElement.firstChild;
		var newNode;
		while (ruleContent) {
			if (ruleContent.nodeName == "xmlscript") {
				newNode = document.createElementNS("http://www.w3.org/1999/xhtml","script");
				newNode.setAttribute("id","toolscript"); //TODO copy all attrs.
				newNode.setAttribute("type", "text/javascript");
				newNode.setAttribute("src", ruleContent.getAttribute("src"));
			}
			else
				newNode = ruleContent.cloneNode(true);
			// locate and replace attribute parameterized values.
			var attrList = this.evaluateXPath(newNode,".//@*[contains(.,'?')]" );
			var attr;
			while ((attr = attrList.iterateNext())) {
				var attrVal = attr.value;
				// handle parameter substitution
				var p = attrVal.indexOf("?$");
				if (p != -1) 
					attr.value = this.substituteParams(attrVal);
				// handle attribute value substitution
				var p = attr.value.indexOf("?");
				if (p != -1) 
					attr.value = this.substituteAttributes(node, attrVal);
			}
			if (newNode.nodeName == "script") {
				document.documentElement.appendChild(newNode);
			}
			else {
				//alert(attrList.length);
				if (!insertAfter) 
					insertPoint.appendChild(newNode);
				else {
					if (currentInsertPoint == insertPoint)
						insertPoint.parentNode.replaceChild(newNode, insertPoint);
					else
						currentInsertPoint.parentNode.insertBefore(newNode, currentInsertPoint.nextSibling);
					currentInsertPoint = newNode;
				}
			}	
			
			if (xmlrule.hasApplies) {
				if (newNode.localName == "xmlapply") {
					this.buildFromApply(node, newNode, newNode, true);
				}
				else {
					var xmlapplies = newNode.getElementsByTagName("xmlapply");
					for (var j=0; j<xmlapplies.length; j++) {
						var xmlapply = xmlapplies.item(j);
						this.buildFromApply(node, xmlapply, xmlapply, true);
					}
				}	
			}
			ruleContent = ruleContent.nextSibling;
		}
	}	
	
}
XMLTemplate.prototype.evaluateXPath = function(contextNode, path) {
	return this.XPATH.evaluate(path, contextNode, this.NSResolver, this.ORDERED_NODE_ITERATOR_TYPE, null);	
}
XMLTemplate.prototype.substituteParams = function(val) {
	for (var paramId in this.xmlparams) {
		var param = this.xmlparams[paramId];
		if (param.id) {
			val = val.replace(param.re, param.value);
		}	
	}
	return val;
}
XMLTemplate.prototype.substituteAttributes = function(node,val) {
	//TODO use XPATH to process attribute value string.
	// Allows access to parent element attributes and XPATH functions.
	do {
		var attrs = this.attributeFinderRE.exec(val);
		if (attrs) {
			var subst = "";
			if (attrs[1].substr(1,1) == "@") {
				var attrName = this.attributeExtractorRE.exec(attrs[1])[1];
				subst = node.getAttribute(attrName) || "";
			}
			val = val.replace(this.attributeFinderRE, subst);
		}	
	} while (attrs);
	return val;
}

XMLTemplate.prototype.buildRules = function() {
	this.xmlrules = [];
	var ruleList = this.xmlTemplate.getElementsByTagName("xmlrule");
	for (var i=0; i<ruleList.length; i++) {
		var xmlrule = {};
		xmlrule.element = ruleList.item(i);
		var applyList = xmlrule.element.getElementsByTagName("xmlapply");
		xmlrule.hasApplies = (applyList.length > 0);
		this.xmlrules[xmlrule.element.getAttribute("match")] = xmlrule;
	}
}
XMLTemplate.prototype.buildParams = function() {
 	this.paramsPresent = false;
	var paramList = this.xmlTemplate.getElementsByTagName("xmlparam");
	this.xmlparams = [];
	for (var i=0; i<paramList.length; i++) {
		var xmlparam = paramList.item(i);
		var param = {};
		param.id = xmlparam.getAttribute("id");		
		param.value = xmlparam.getAttribute("value");
		param.re = new RegExp("\\?\\$" + param.id, "g");		
		this.xmlparams[param.id] = param;
		this.paramsPresent = true;
	}
}
XMLTemplate.prototype.buildElementList = function(xmlnode, applyElement) {
	var path = applyElement.getAttribute("path");
	assert(path, "path attribute not found on xmlapply element in xmltemplate " + this.id + ".");
	if (this.paramsPresent)
		path = this.substituteParams(path);

	return this.evaluateXPath(xmlnode, path);
}
