/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
 
var RE_Whitespace = /^\s*$/;

function XMLTreeView(dom)
{
  // columnids is an array of strings indicating the names of the columns, in order
  this.dom = dom;
  this.viewData = new ViewData(dom, 0, true);
  this.viewData.isContainerOpen = true;
  this.viewData.loadChildren();
  this.tree = null;
  this.data = new Array;
  this.selection = null;
  this.cachedRow = -111;
  this.cachedViewData = null;
}
XMLTreeView.prototype.__defineGetter__('rowCount', function() {
	return this.viewData.childRows;
})
XMLTreeView.prototype.__defineSetter__('rowCount', function(c) {
	throw "rowCount is a readonly property";
})
XMLTreeView.prototype.setTree = function(tree) {
   this.tree = tree;
}
XMLTreeView.prototype.getCellText = function(row, column) {
    var viewData = this.getViewDataAtRow(row); 
	 //debug("getText:" + row + " " + viewData.formatNode(this));
    return viewData.formatNode(this) + row;
}
XMLTreeView.prototype.setCellText = function(row, column, value) {
}
XMLTreeView.prototype.addRow = function(row) {
    var oldrowcount = this.rows;
    this.rows = this.data.push(row);
}
XMLTreeView.prototype.addRows = function(rows) {
	var oldrowcount = this.rowCount;

	var length = rows.length;
	for(var i = 0; i < length; i++)
		this.rows = this.data.push(rows[i]);
}
XMLTreeView.prototype.rowCountChanged = function(row, count) {
	this.tree.rowCountChanged(row, count);
}
XMLTreeView.prototype.invalidate = function() {
	this.tree.invalidate();
}
XMLTreeView.prototype.clear = function() {
	this.data = null;
}
XMLTreeView.prototype.getRowProperties = function(row, column, prop) { }
XMLTreeView.prototype.getCellProperties = function(row, prop) { }
XMLTreeView.prototype.getColumnProperties = function(column, elem, prop) { }
XMLTreeView.prototype.isContainer = function(row) {
	return this.getViewDataAtRow(row).isContainer;
}
XMLTreeView.prototype.isContainerOpen = function(row) {
	return this.getViewDataAtRow(row).isContainerOpen;
}
XMLTreeView.prototype.isContainerEmpty = function(row) { return false; }
XMLTreeView.prototype.isSeparator = function(row) { return false; }
XMLTreeView.prototype.isSorted = function() { return false; }
XMLTreeView.prototype.canDropOn = function(row) { return false; }
XMLTreeView.prototype.canDropBeforeAfter = function(row, before) { return false; }
XMLTreeView.prototype.drop = function(row, orientation) { return false; }
XMLTreeView.prototype.getParentIndex = function(row) { return 0; }
XMLTreeView.prototype.hasNextSibling = function(row, after) { return false; }
XMLTreeView.prototype.getLevel = function(row) {
	try {
		var viewData = this.getViewDataAtRow(row);
		return viewData.getLevel();
	}
	catch (e) {
		return 0;
	}
}
XMLTreeView.prototype.cycleHeader = function(col, elem) { }
XMLTreeView.prototype.selectionChanged = function() { }
XMLTreeView.prototype.cycleCell = function(row, column) { }
XMLTreeView.prototype.isEditable = function(row, column) { return false; }
XMLTreeView.prototype.performAction = function(action) { }
XMLTreeView.prototype.performActionOnRow = function(action, row) { }
XMLTreeView.prototype.performActionOnCell = function(action, row, column) { }
XMLTreeView.prototype.toggleOpenState = function(row) {
	//debug("toggle");  		
	var viewData = this.getViewDataAtRow(row);
	var delta;
	if (!viewData.isContainerOpen) {
		if (!viewData.firstChild)
			viewData.loadChildren();
		delta = viewData.childRows; 
		viewData.isContainerOpen = true;
	}
	else {
		delta = -viewData.childRows; 	
		viewData.isContainerOpen = false;
	}	
	viewData.updateChildRows(delta);
	this.rowCountChanged(row, delta);
}
XMLTreeView.prototype.getViewDataAtRow = function(row) {
	// use cached value if present.
	if (row == this.cachedRow) 
		return this.cachedViewData;
	this.cachedRow = row;
	this.cachedViewData = this.getViewDataAtRowInternal(this.viewData, row, row);
	return this.cachedViewData;
}
XMLTreeView.prototype.getViewDataAtRowInternal = function(parentViewData, row, count) {
	var viewData = parentViewData.firstChild;
	while (viewData) {
			//debug("getN:" + viewData.node.nodeType + " row:" + row + " count:" + count + " childRows:" + viewData.childRows);
			count -= viewData.rowHeight;
			if (count < 0 )
				return viewData;
			var childRows = (viewData.isContainerOpen) ? viewData.childRows : 0;
			if (count < childRows)
				return this.getViewDataAtRowInternal(viewData, row, count);
			count -= childRows;	
			if (count < 0 )
				return viewData;
		viewData = viewData.nextSibling;
	}
}

function ViewData(node, rowHeight, isContainer) {
	this.parent = null;
	this.nextSibling = null;	
	this.previousSibling = null;
	this.firstChild = null;
	this.lastChild = null;
	
	this.node = node;
	this.rowHeight = rowHeight;
	this.childRows = 0;
	this.isContainer = isContainer;
	this.isContainerOpen = false;
}

ViewData.prototype.remove = function() {
	if (this.parent) {
		// unlink viewData from sibling chain.
		if (this.previousSibling) 
			this.previousSibling.nextSibling = this.nextSibling; 			
		if (this.nextSibling) 
			this.nextSibling.previousSibling = this.previousSibling;
		// now unhook from parent if first or last in child chain.	
		if (this.parent.firstChild == this)
			this.parent.firstChild = this.nextSibling;
		if (this.parent.lastChild == this)
			this.parent.lastChild = this.previousSibling;
		this.parent = null;			
	}
	this.updateChildRows(-(newViewData.rowHeight + newViewData.childRows));
	return this;
}

ViewData.prototype.replace = function(newViewData) {
	if (newViewData.parent)
		newViewData.remove();
	newViewData.parent = this.parent;
	newViewData.nextSibling = this.nextSibling;		
	newViewData.previousSibling = this.previousSibling;
	if (this.previousSibling)
		this.previousSibling.nextSibling = newViewData;
	if (this.nextSibling)
		this.nextSibling.previousSibling = newViewData;
	if (this.parent) {
		if (this.parent.firstChild == this)
			this.parent.firstChild = newViewData;
		if (this.parent.lastChild == this)
			this.parent.lastChild = newViewData;
	}
	this.parent = null;
	return this;
}

ViewData.prototype.insertAfter = function(newViewData) {
	if (newViewData.parent)
		newViewData.remove();
	newViewData.nextSibling = this.nextSibling;
	newViewData.previousSibling = this;
	this.nextSibling = newViewData;
	if (this.parent) {
		if (this.parent.lastSibling == this)
			this.parent.lastSibling = newViewData;
	}
	newViewData.updateChildRows(newViewData.rowHeight + newViewData.childRows);
}

ViewData.prototype.append = function(viewData) {
	assert(viewData, "ViewData undefined for append");
	if (viewData.parent)
		viewData.remove();
	viewData.parent = this;
	if (this.lastChild) {
		viewData.previousSibling = this.lastChild;
		this.lastChild.nextSibling = viewData;
		this.lastChild = viewData;
	}
	else {
		this.firstChild = viewData;
		this.lastChild = viewData;
	}
	viewData.updateChildRows(viewData.rowHeight + viewData.childRows);
}
ViewData.prototype.updateChildRows = function(delta) {
	var viewData = this.parent;
	while (viewData) {
		viewData.childRows += delta;
		if (!viewData.isContainerOpen)
			break;
		viewData = viewData.parent;
	}
}
ViewData.prototype.getLevel = function() {
	var level = 0;
	var viewData = this.parent;
	while (viewData) {
		level++;
		viewData = viewData.parent;
	}
	return level; 
}



ViewData.prototype.loadChildren = function() {
	var i = 0;
	var node = this.node.firstChild;	
	while (node) {
		switch (node.nodeType) {
		case NODE_TYPE_ELEMENT:
			this.append(new ViewData(node, 1, true));
			//debug("prepare EL:" + node.localName);
			break;
		case NODE_TYPE_TEXT:	
		case NODE_TYPE_CDATA:
			var isWS = RE_Whitespace.test(node.nodeValue);
			if (!isWS) {
				this.append(new ViewData(node, 1, false));
				//debug("prepare TX:" + node.nodeValue);
			}
			break;
		case NODE_TYPE_PROCESSING_INSTRUCTION:
			this.append(new ViewData(node, 1, false));
			//debug("prepare PI:" + node.name);
			break;
		case NODE_TYPE_COMMENT:
			this.append(new ViewData(node, 1, false));
			break;
		case NODE_TYPE_DOCUMENT_TYPE:
			this.append(new ViewData(node, 1, false));
			//debug("prepare DT:" + node.name);
			break;
		default:
			break;
		}	
		node = node.nextSibling;
	}
}

ViewData.prototype.formatNode = function(view) {
	var node = this.node;
	switch (node.nodeType) {
	case NODE_TYPE_ELEMENT:
		var str = "<"+ node.localName
		var attrs = node.attributes;	
		for (var i=0; i < attrs.length; i++) {
			var attr = attrs.item(i);
			str += " " + attr.name + '="' + attr.value + '"';
		}
		str += ">";
		return str;
	case NODE_TYPE_TEXT:	
	case NODE_TYPE_CDATA:
		return node.nodeValue;
	case NODE_TYPE_ATTRIBUTE:
		return node.nodeValue;
	case NODE_TYPE_PROCESSING_INSTRUCTION:
		return "<?" + node.target + " " + node.data + "?>";
	case NODE_TYPE_COMMENT:
		return "<!-- " + node.data + "-->";
	case NODE_TYPE_DOCUMENT_TYPE:
		return "<!DOCTYPE " + node.name + ">";
	default:
		return "other";
	}	
}

