/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */

netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");

const START = 0;
const WHITESPACE = 1;
const NEWLINE = 2;
const WORD = 3;
const RESERVED = 4
const BRACKET = 5;
const PUNCTUATION = 6;		
const QUOTE = 7;		// string
const APOSTROPHE = 8; // string
const OPERATOR = 9;
const COMMENT = 10; // comment
const BLOCKCOMMENT = 11; /* comment */
const BLOCKCOMMENT_END = 12; /* comment */

const CHR_SPACE 		= 32; 
const CHR_TAB			= 9;
const CHR_FF			= 12;	// Form feed 		
const CHR_LF			= 10;	// Line feed
const CHR_CR			= 13;	// Carriage return
const CHR_QUOTE 		= 34;	// "
const CHR_APOSTROPHE 	= 39;	// '
const CHR_LPAREN		= 40;	// (
const CHR_RPAREN		= 41;	// )
const CHR_LBRACE		= 123;	// {
const CHR_RBRACE		= 125;	// }
const CHR_LBRACKET		= 91;	// [
const CHR_RBRACKET		= 93;	// ]
const CHR_FWDSLASH		= 47;	// /
const CHR_STAR 			= 42;	// *
const CHR_EXCLAMATION	= 33;	// ! 
const CHR_EQUAL			= 61;	// = 
const CHR_PLUS			= 43;	// + 
const CHR_MINUS			= 45;	// - 
const CHR_DASH			= 45;	// - 
const CHR_AMP			= 38;	// & 
const CHR_BAR			= 124;	// ! 
const CHR_GT			= 62;	// > 
const CHR_LT			= 60;	// < 
const CHR_TILDE			= 126;	// ~ 
const CHR_PERCENT		= 37;	// % 
const CHR_QUESTION		= 63;	// ?
const CHR_COMMA			= 44;	// ,
const CHR_STOP			= 46;	// .		
const CHR_COLON			= 58;	// :
const CHR_SEMICOLON		= 59;	// ;

var className = ["start", "whitespace", "newline", "word", "reserved", "bracket", "punctuation", "quote", "apostrophe", "operator", "comment", "blockcomment"]

var reservedWords = new Object();
reservedWords["xml-stylesheet"] = 1;
reservedWords["xml-overlay"] = 1;
// note will also pick up methods and properties of "Object".



var tokenCount = 0;

function tokenize(text, tokens, tokenTypes) {
	tokenCount = 0;
	var ch;
	var nextCh;
	var token = "";
	var charClass;
	var inQuote = false;
	var inComment = false;
	var state = START;
	var t = 0;
	var t1 = 0; // Token start offset within text
	var len = text.length;
	while ( t < len ) {
		ch = text.charCodeAt(t);
		// assign character class to character
		switch (ch) {
		case CHR_SPACE:
		case CHR_TAB: 
		case CHR_FF: 		
			charClass = WHITESPACE;
			break;
		case CHR_LF:	
		case CHR_CR:
			charClass = NEWLINE;
			break;
		case CHR_LT:
			nextCh = text.charCodeAt(t+1);
			switch (nextCh) {
			case CHR_FWDSLASH:
				charClass = END_BLOCK_ELEM_OPEN;
				break;
			case CHR_QUESTION:
				charClass = START_PI;
				break;
			case CHR_EXCLAMATION:
				charClass = START_COMMENT;
				break;
			default:
				charClass = START_ELEM_OPEN;
			}
			break;
		case CHR_QUESTION:
		case CHR_DASH:
		case CHR_FWDSLASH:
			nextCh = text.charCodeAt(t+1);
			if (nextCh == CHR_GT) {
				case CHR_FWDSLASH:
					charClass = START_ELEM_CLOSE;
					break;
				case CHR_QUESTION:
					charClass = PI;
					break;
				case CHR_EXCLAMATION:
					charClass = COMMENT;
					break;
				default:
					charClass = START_TAG_OPEN;
				}
			}
			break;
		case CHR_GT:
			chatClass = BLOCK_ELEM_
			break;
		case CHR_APOSTROPHE:
			charClass = APOSTROPHE;
			break;
		case CHR_FWDSLASH:
			nextCh = text.charCodeAt(t+1);
			switch (nextCh) {
			case CHR_STAR:
				charClass = BLOCKCOMMENT;
				break;
			case CHR_FWDSLASH:	
				charClass = COMMENT;
				break;
			default:
				charClass = OPERATOR; 				
			}
			break;
		case CHR_STAR:
			nextCh = text.charCodeAt(t+1);
			if (nextCh == CHR_FWDSLASH) {
				charClass = BLOCKCOMMENT_END;
			}	
			else	
				charClass = OPERATOR;  
			break;
		case CHR_EXCLAMATION: 
		case CHR_EQUAL: 
		case CHR_PLUS: 
		case CHR_MINUS: 
		case CHR_AMP: 
		case CHR_BAR: 
		case CHR_GT: 
		case CHR_LT: 
		case CHR_TILDE: 
		case CHR_PERCENT: 
		case CHR_QUESTION:
			charClass = OPERATOR;
			break;
		case CHR_STOP:
		case CHR_COMMA:		
		case CHR_COLON:
		case CHR_SEMICOLON:
			charClass = PUNCTUATION;
			break;
		default:
			charClass = WORD;
		}

		switch (state) {
		case START:
			state = charClass;	
			break;
		case WORD:
		case WHITESPACE:
		case OPERATOR:
		case BRACKET:			
		case PUNCTUATION:						
			if (charClass != state) {
				t1 = addToken(state,text, t1, t);
				state = charClass;
			}
			else {
				t++;
			}	
			break;
		case QUOTE:
		case APOSTROPHE:		
			if (!inQuote){
				inQuote = true;
				t1 = addToken(state,text, t1, t);
				t++;
			}
			else {
				if (charClass == state) {
					inQuote = false;
					t++;
					t1 = addToken(state,text, t1, t);
					state = START;
					t++;
				}	
				else	
					t++;
			}	
			break;
		case NEWLINE:
			addToken(NEWLINE,"\n", 0, 1);
			t++
			nextCh = text.charCodeAt(t);
			if (nextCh == CHR_LF) 
				t++; 	
			t1 = t;
			state = START;
			break;
		case COMMENT:
			if (!inComment) {
				t++; // skip second slash.
				inComment = true;
			}
			if (charClass == NEWLINE) {
				inComeent = false;
				t1 = addToken(state,text, t1, t);
				state = NEWLINE;
			}
			else {
				t++;
			}	
			break;
		case BLOCKCOMMENT:
			switch (charClass) {
			case NEWLINE:
				addToken(state,text, t1, t);
				addToken(NEWLINE,"\n", 0, 1);
				t++;
				nextCh = text.charCodeAt(t);
				if (nextCh == CHR_LF) 
					t++;
				t1 = t;	

				break
			case BLOCKCOMMENT_END:
				t++; // skip following slash
				t1 = addToken(state, text, t1, t + 1);
				state = START;
				break;
			default:	
				t++;
			}	
			break;
		}
	}
	addToken(state,text, t1, t);

	return tokenCount;
}

function addToken(state, text, start, end) {
	var token = text.substring(start,end);
	//debug("token:" + token);
	if (token.length > 0) {
		if (state == WORD) {
			if ((token) in reservedWords )
				state = RESERVED;
		}
		if (tokenCount > tokens.length) {
			tokens.push(token);
			tokenTypes.push(state);
		}	
		else {
			tokens[tokenCount] = token;
			tokenTypes[tokenCount] = state;
		}	
		tokenCount++;
	}	


	return end;
}

