/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
 
const tabs = "\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t";
var nl; 

function XULDOMSerializer(xuloutput) {
	this.longAttributeLength = xuloutput.longAttributeLength;
	this.maxAttributesPerLine = xuloutput.maxAttributesPerLine;
	this.generateDTDSave = xuloutput.generateDTD;		
	this.generateDTD = xuloutput.generateDTD;		
	this.generateDTDEntityFor = xuloutput.generateDTDEntityFor;
	this.XML = null;
	this.DTD = null;
	this.entityNames = null;
	this.entityValues = null;
	this.notWhitespace = new RegExp(/\S/);
	nl = Utility.NEWLINE;
	this.CTR_EMPTY		= 0;
	this.CTR_INLINE	= 1;
	this.CTR_BLOCK		= 2; 
	this.internalSubset = null;
}


XULDOMSerializer.prototype.serialize = function(dom) {
	this.internalSubset = null;
	this.XML = '<?xml version="1.0"?>';
	this.generateDTD = this.generateDTDSave;
	var docType = Utility.getDocType(dom);
	// even though DTD generation is switched on, this doc may not have what it takes.
	this.generateDTD = (this.generateDTD && docType && docType.systemId && docType.systemId.substr(0,7) == "chrome:");
	if (this.generateDTD) {
		this.entityNames = new Array();
		this.entityValues = new Array();
	}	
	else {
		this.entityNames = null;
		this.entityValues = null;
	}
	var nodes = dom.childNodes;
	for (var i=0; i<nodes.length; i++) {
		var containerType = this.getContainerType(nodes.item(i));
		this.serializeNode(nodes.item(i), containerType, 0);
	}
	
	if (this.generateDTD) {
		this.serializeDTD();
	}
	else
		this.DTD = null;
	this.entityNames = null; // nolonger of use.
	this.entityValues = null; // nolonger of use.
}
// a recursive function to re/build the DOM tree
XULDOMSerializer.prototype.serializeNode = function(node, containerType, level) {
	var innerCT = this.getContainerType(node);
	var isEmpty = (innerCT == this.CTR_EMPTY);
	this.serializeNodeStart(node, level, containerType, isEmpty);
	if (!isEmpty) {
		for (var i = 0; i < node.childNodes.length; i++) {
			this.serializeNode(node.childNodes.item(i), 
						((containerType==this.CTR_INLINE)?this.CTR_INLINE:innerCT),
						level + 1);
		}
	}
	this.serializeNodeEnd(node, level, innerCT, isEmpty);
}
XULDOMSerializer.prototype.serializeNodeStart = function(node, level, containerType, isEmpty) {
	switch (node.nodeType) {
	case NODE_TYPE_ELEMENT:
		if (containerType != this.CTR_INLINE)
			this.XML += nl + this.indent(level);
		this.XML += "<" + node.nodeName;
		this.serializeElementAttributes(node, level);
		if (isEmpty) 
			this.XML += "/>";
		else 
			this.XML += ">";
		break;
	case NODE_TYPE_COMMENT:
		if (containerType != this.CTR_INLINE)
			this.XML += nl + this.indent(level);
		//TODO: convert nl and cr in comment data.
		this.XML += "<!--" + node.data;
		break;
	case NODE_TYPE_TEXT:
		if (this.notWhitespace.test(node.nodeValue)) {
			this.XML += node.nodeValue;
		}	
		break;
	case NODE_TYPE_CDATA:
		this.XML += nl + "<![CDATA[" + node.data + "]]>";
		break;
	case NODE_TYPE_PROCESSING_INSTRUCTION:
		if (node.target == "xulapp-internalsubset")
			this.internalSubset = node.data;
		else {	
			this.XML += nl + this.indent(level);
			this.XML += "<?" + node.target + " " + node.data;
		}	
		break;
	case NODE_TYPE_DOCUMENT_TYPE:
		this.XML += nl + this.indent(level);
		this.XML += "<!DOCTYPE " + node.name;
		if (node.publicId != "")
			this.XML +=  ' PUBLIC "' + this.encodeValue(node.publicId) + '" "' + this.encodeValue(node.systemId) + '"';
		else {	
			if (node.systemId != "")
				this.XML +=  ' SYSTEM "' + this.encodeValue(node.systemId) + '"';
		}		
		if (this.internalSubset)
			this.XML += " " + this.internalSubset;
		break;
	}
}

XULDOMSerializer.prototype.serializeElementAttributes = function(node, level) {
	var attrsOnLine = 0;
	for (var i=0; i < node.attributes.length; i++) {
		var attr = node.attributes.item(i);
		if (i > 0 && attr.value.length > this.longAttributeLength || attrsOnLine > this.maxAttributesPerLine) {
			this.XML += nl + this.indent(level + 1);
		}	
		this.XML += " " + attr.name + '="' + this.buildAttrValue(node, attr) + '"';
		attrsOnLine++;
	}
}

XULDOMSerializer.prototype.serializeNodeEnd = function(node, level, containerType, isEmpty) {
	switch (node.nodeType) {
	case NODE_TYPE_ELEMENT:
		if (!isEmpty) {	
			if (containerType == this.CTR_BLOCK)
				this.XML += nl + this.indent(level);
			this.XML += "</" + node.nodeName + ">";
		}	
		break;
	case NODE_TYPE_COMMENT:
		//TODO: convert nl and cr in comment data.
		this.XML += "-->";
		break;
	case NODE_TYPE_PROCESSING_INSTRUCTION:
		if (node.target != "xulapp-internalsubset")
			this.XML += "?>";
		break;
	case NODE_TYPE_DOCUMENT_TYPE:
		this.XML += ">";
		break;
	}
}
XULDOMSerializer.prototype.getContainerType = function(node) {
	var containerType = this.CTR_EMPTY;
	node = node.firstChild;
	while (node) {
		switch (node.nodeType) {
			case NODE_TYPE_ELEMENT:
				containerType |= this.CTR_BLOCK;
				break;
			case NODE_TYPE_TEXT:	
				if (this.notWhitespace.test(node.nodeValue))
					return this.CTR_INLINE;
				break;
			case NODE_TYPE_CDATA:	
				return this.CTR_BLOCK;
				break;
			case NODE_TYPE_COMMENT:	
				return this.CTR_BLOCK;
			default:
				return this.CTR_BLOCK;
		}			
		node = node.nextSibling;
	}
	return containerType;
}


XULDOMSerializer.prototype.indent = function(level) {
	return tabs.substr(0,level);
}

XULDOMSerializer.prototype.buildAttrValue = function(node, attr) {
	var val = this.encodeValue(attr.value);
	if (this.generateDTD) {
		var searchKey = node.nodeName + "." + attr.name;
		var genericSearchKey = "*." + attr.name;
		for (var i=0; i < this.generateDTDEntityFor.length; i++) {
			var matchKey = this.generateDTDEntityFor[i];
			if (matchKey == genericSearchKey || matchKey == searchKey) {
				var id = this.getId(node, attr); // id is some unique identifier for this
				if (id) {
					var entityName = node.nodeName + "." + id + "." + attr.name; //TODO attr.name may contain NS
					this.entityNames[this.entityNames.length] = entityName;
					this.entityValues[this.entityValues.length] = val;
					val = "&" + entityName + ";";
				}	
				break;
			}
		}
	}
	return val;
}

XULDOMSerializer.prototype.encodeValue = function(val) {
	val = val.replace(/[&]/g,"&amp;");
	val = val.replace(/</g,"&lt;");
	val = val.replace(/>/g,"&gt;");
	val = val.replace(/\"/g,"&quot;"); //"fool my text editor syntax checker into thinking the open string has ended.
	return val;
}

XULDOMSerializer.prototype.getId = function(node, attr) {
	if (node.hasAttribute("id")) // do we have a unique id?
		return node.getAttribute("id"); 
	if (node.hasAttribute("control")) { // do we reference a unique id.
		return node.getAttribute("control"); // TODO check entityNames if has been used already.
	}
	//TODO allocate a unique id and create the attribute.
	return null;	
}

XULDOMSerializer.prototype.serializeDTD = function() {
	this.DTD = "";
	for (var i=0; i < this.entityNames.length; i++) {
		this.DTD += "<!ENTITY " + this.entityNames[i] + ' "' + this.encodeValue(this.entityValues[i]) + '">' + nl;
	}
}
