/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
 
// Create a tree view which aggregates the supplied content tree view.
// the canDrop and drop fucntions can then be added at will.

const  DROP_BEFORE = Components.interfaces.nsITreeView.inDropBefore;
const  DROP_ON = Components.interfaces.nsITreeView.inDropOn;
const  DROP_AFTER = Components.interfaces.nsITreeView.inDropAfter;

const PROGRESS_NORMAL = 1;
const PROGRESS_UNDETERMINED = 2;
const PROGRESS_NONE = 3;


//interface nsITreeView
function DroppableTreeView(view) {
	this.view = view;
}
DroppableTreeView.prototype.__defineGetter__("wrappedJSObject", function()
{

	return this;
})

// nsITreeContentView==========================================
DroppableTreeView.prototype.getItemAtIndex = function(index) {
	// Warning this only return index of relative to visible/expanded items.
	return this.view.getItemAtIndex(index);
}
DroppableTreeView.prototype.getIndexOfItem = function(item) {
	return this.view.getIndexOfItem(item);
}

// nsITreeView==========================================
//====================================================================

/**
  * The total number of rows in the tree (including the offscreen rows).
  */
DroppableTreeView.prototype.__defineGetter__("rowCount", function()
{

	return this.view.rowCount;
})

/**
  * The selection for this view.
  */
DroppableTreeView.prototype.__defineGetter__("selection", function()
{
	return this.view.selection;
})
DroppableTreeView.prototype.__defineSetter__("selection", function(val)
{
	this.view.selection = val;
})

/** 
  * An atomized list of properties for a given row.  Each property, x, that
  * the view gives back will cause the pseudoclass :moz-tree-row-x
  * to be matched on the pseudoelement ::moz-tree-row.
  */
DroppableTreeView.prototype.getRowProperties = function(index, properties) {
	if (this.view.getRowProperties)
		return this.view.getRowProperties(index, properties);
	else
		return null;
}

/**
  * An atomized list of properties for a given cell.  Each property, x, that
  * the view gives back will cause the pseudoclass :moz-tree-cell-x
  * to be matched on the ::moz-tree-cell pseudoelement.
  */
DroppableTreeView.prototype.getCellProperties = function(row, col, properties) {
	if (this.view.getCellProperties)
		return this.view.getCellProperties(row, col, properties);
	else
		return null;
}
  
/**
  * Called to get properties to paint a column background.  For shading the sort
  * column, etc.
  */
DroppableTreeView.prototype.getColumnProperties = function(col, colElem, properties)
{
	if (this.view.getColumnProperties)
		return this.view.getColumnProperties(col, colElem, properties);
	else
		return null;
}

/**
  * Methods that can be used to test whether or not a twisty should be drawn,
  * and if so, whether an open or closed twisty should be used.
  */
DroppableTreeView.prototype.isContainer = function(index) {
	return this.view.isContainer(index);
}
DroppableTreeView.prototype.isContainerOpen = function(index) {
	return this.view.isContainerOpen(index);
}
DroppableTreeView.prototype.isContainerEmpty = function(index) {
	return this.view.isContainerEmpty(index);
}
 
/**
  * isSeparator is used to determine if the row at index is a separator.
  * A value of true will result in the tree drawing a horizontal separator.
  * The tree uses the ::moz-tree-separator pseudoclass to draw the separator.
  */
DroppableTreeView.prototype.isSeparator = function(index) {
	return this.view.isSeparator(index);
}

/**
  * Specifies if there is currently a sort on any column. Used mostly by dragdrop
  * to affect drop feedback.
  */
DroppableTreeView.prototype.isSorted = function() {
	return this.view.isSorted();
}
/**
  * Methods used by the drag feedback code to determine if a drag is allowable at
  * the current location. To get the behavior where drops are only allowed on
  * items, such as the mailNews folder pane, always return false when
  * the orientation is not DROP_ON.
  */
DroppableTreeView.prototype.canDrop = function(index, orientation) {
// Not sure if this is used any more?
	return false;
}
//=============================================================
// Override these functions to provide drag drop support.
DroppableTreeView.prototype.canDropOn = function(index) {
	return false;
}

DroppableTreeView.prototype.canDropBeforeAfter = function(index, before) {
	return false;
}
/**
   * Called when the user drops something on this view. The |orientation| param
   * specifies before/on/after the given |row|.
   */
DroppableTreeView.prototype.drop = function(index, dropWhere) {
}
  
/**
  * Methods used by the tree to draw thread lines in the tree.
  * getParentIndex is used to obtain the index of a parent row.
  * If there is no parent row, getParentIndex returns -1.
  */
DroppableTreeView.prototype.getParentIndex = function(index) { 
	return this.view.getParentIndex(index);
}

/**
  * hasNextSibling is used to determine if the row at rowIndex has a nextSibling
  * that occurs *after* the index specified by afterIndex.  Code that is forced
  * to march down the view looking at levels can optimize the march by starting
  * at afterIndex+1.
  */
DroppableTreeView.prototype.hasNextSibling = function(index, afterIndex) { 
	return this.view.hasNextSibling(index, afterIndex);
}

/**
  * The level is an integer value that represents
  * the level of indentation.  It is multiplied by the width specified in the 
  * :moz-tree-indentation pseudoelement to compute the exact indendation.
  */
DroppableTreeView.prototype.getLevel = function(index) { 
	return this.view.getLevel(index);
}

/**
  * The image path for a given cell. For defining an icon for a cell.
  * If the empty string is returned, the :moz-tree-image pseudoelement
  * will be used.
  */
DroppableTreeView.prototype.getImageSrc = function(index, col) { 
	return this.view.getImageSrc(index, col);
}

/**
  * The progress mode for a given cell. This method is only called for
  * columns of type |progressmeter|.
  */
DroppableTreeView.prototype.getProgressMode = function(index, col) { 
	return this.view.getProgressMode(index, col);
}

/**
  * The value for a given cell. This method is only called for columns
  * of type other than |text|.
  */
DroppableTreeView.prototype.getCellValue = function(index, col) { 
	return this.view.getCellValue(index, col);
}

/**
  * The text for a given cell.  If a column consists only of an image, then
  * the empty string is returned.  
  */
DroppableTreeView.prototype.getCellText = function(index, col) { 
	return this.view.getCellText(index, col);
}

/**
  * Called during initialization to link the view to the front end box object.
  */
DroppableTreeView.prototype.setTree = function(tree) { 
	this.view.setTree(tree);
}
/**
  * Called on the view when a header is clicked.
  */
DroppableTreeView.prototype.cycleHeader = function(col) { 
	this.view.cycleHeader(col);
}  

/**
  * Should be called from a XUL onselect handler whenever the selection changes.
  */
DroppableTreeView.prototype.selectionChanged = function() { 
	this.view.selectionChanged();
}

/**
  * Called on the view when a cell in a non-selectable cycling column (e.g., unread/flag/etc.) is clicked.
  */
DroppableTreeView.prototype.cycleCell = function(index, col) { 
	this.view.cycleCell(index, col);
}
  
/**
  * isEditable is called to ask the view if the cell contents are editable.
  * A value of true will result in the tree popping up a text field when 
  * the user tries to inline edit the cell.
  */
DroppableTreeView.prototype.isEditable = function(index, col) { 
	return this.view.isEditable(index,col);
}

/**
  * Called on the view when an item is opened or closed.
  */
DroppableTreeView.prototype.toggleOpenState = function(index) { 
	this.view.toggleOpenState(index);
}


/**
  * setCellValue is called when the value of the cell has been set by the user.
  * This method is only called for columns of type other than |text|.
  */
DroppableTreeView.prototype.setCellValue = function(row, col, val) { 
	this.view.setCellValue(row,col, val);
}

/**
  * setCellText is called when the contents of the cell have been edited by the user.
  */   
DroppableTreeView.prototype.setCellText = function(row, col, val) { 
	this.view.setCellText(row,col, val);
}

/**
  * A command API that can be used to invoke commands on the selection.  The tree
  * will automatically invoke this method when certain keys are pressed.  For example,
  * when the DEL key is pressed, performAction will be called with the "delete" string.
  */
DroppableTreeView.prototype.performAction = function(action) { 
	this.view.performAction(action);
}

/**
  * A command API that can be used to invoke commands on a specific row.
  */
DroppableTreeView.prototype.performActionOnRow = function(action, row) { 
	this.view.performActionOnRow(action, row);
}

/**
  * A command API that can be used to invoke commands on a specific cell.
  */
DroppableTreeView.prototype.performActionOnCell = function(action, index, col) { 
	this.view.performActionOnCell(action, index, col);
}
