/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
 
netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");

var xulapp = window.arguments[0];
var OKCallback = window.arguments[1];

var projectName; // Element ref.
var parentDirectoryName; // Element ref.
var projectDisplayName; // Element ref.
var projectAuthor; // Element ref.

function onload_window(event) {
	projectName = document.getElementById("projectName");
	parentDirectoryName = document.getElementById("parentDirectoryName");
	projectDisplayName = document.getElementById("projectDisplayName");
	projectAuthor = document.getElementById("projectAuthor");
	window.sizeToContent();
}

function checkProjectName() {
	//TODO parse valid project name.
	var re = new RegExp(/^[a-z0-9]+$/);
	if (!re.test(projectName.value)) {
		alert(xulapp.getMsg("NEWPRJ.ProjectNameFormat"));
		projectName.focus();
		return false;
	}
	return true;
}

function checkParentDirectoryName() {
	if (parentDirectoryName.value && parentDirectoryName.value != "") {
		var directory = new File(parentDirectoryName.value);
		return checkParentDirectory(directory);
	}	
	else {
		alert(xulapp.getMsg("NEWPRJ.DirectoryRequired"));
		parentDirectoryName.focus();
		return false;
	}	
}

function checkParentDirectory(directory) {
	if (!directory.exists()) {
		alert(xulapp.getMsg("NEWPRJ.DirectoryNotExists",directory.path));
		parentDirectoryName.focus();
		return false;
	}
	if (!directory.isDirectory()) {
		alert(xulapp.getMsg("NEWPRJ.IsNotDirectory",directory.path));
		parentDirectoryName.focus();
		return false;
	}
	if (!directory.isWritable()) {
		alert(xulapp.getMsg("NEWPRJ.DirectoryNotWritable",directory.path));
		parentDirectoryName.focus();
		return false;
	}
	return true;
}

function checkFormValid() {
	var valid = true;
	valid = valid && checkProjectName();
	valid = valid && checkParentDirectoryName();
	return valid; 
}

function oncommand_browse(event) {
	const FP = Components.interfaces.nsIFilePicker;
	var filePicker = Utility.getComponent("@mozilla.org/filepicker;1","nsIFilePicker");
	assert(filePicker,"Unable to obtain file picker.");
	
	filePicker.init(window, "XulApp New Project - Choose parent directory for project", FP.modeGetFolder);
	var filePicked = filePicker.show();
		if (filePicked == FP.returnOK && filePicker.file.path) {
		// load selected project parent directory into textbox.
		var directory;
		directory = new File(filePicker.file.path);
		if (checkParentDirectory(directory)) {
			parentDirectoryName.value = directory.path; 
		}
	}
}

function oncommand_ok(event) {
	if (!checkFormValid()) 
		return;
	var project = new Project();
	if (project.newProject(projectName.value, parentDirectoryName.value, projectDisplayName.value, projectAuthor.value)) {
		xulapp.project = project;
		alert(xulapp.getMsg("NEWPRJ.CreateOK", projectName.value));
		OKCallback(project.projectFileURI);
	}
	else {
		alert(xulapp.getMsg("NEWPRJ.CreateFailed", projectName.value));
	}	
	window.close();
}

function oncommand_cancel(event) {
	window.close();
}

