/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
 
netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");

const promptService = 
	Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
	getService(Components.interfaces.nsIPromptService);
//====================================================================================
// Open Documents ====================================================================
// The collection of open documents. This must be a singleton.

function OpenDocuments(xulapp) {
// properties:
	this.xulapp = xulapp;
	this.openDocs = [];
}

OpenDocuments.prototype.createDocument = function(path, extension, isNew) {
	var doc = new OpenDocument(path, extension, isNew);
	this.addDocument(doc);
	return doc;
}

OpenDocuments.prototype.closeAll = function() {
	// need to cache length as close window shortens array.
	var len = this.openDocs.length;
	for (var i = 0; i < len; i++) {
		this.openDocs[0].close();
	}	
}

OpenDocuments.prototype.addDocument = function(openDoc) {
	this.openDocs.push(openDoc);
	openDoc.openDocuments = this;
}


OpenDocuments.prototype.rmvDocument = function(openDoc) {
	Utility.removeArrayEntry(this.openDocs, openDoc);
	openDoc.openDocuments = null; // Assist GC.
}

OpenDocuments.prototype.findDocumentByPath = function(path) {
	for (var i = 0; i < this.openDocs.length; i++) {
		if (this.openDocs[i].path == path) return this.openDocs[i];
	}
	return null;
}

OpenDocuments.prototype.createIterator = function() {
	return new OpenDocumentsIterator(this.openDocs);
}

OpenDocuments.prototype.__defineGetter__( "length", function() {
	return this.openDocs.length;
})

//====================================================================================
// OpenDocumentsIterator =============================================================
// behavior is not specified if more than one iterator is open simultaneously.
// This constructor should not be called directly - use OpenDocuments.createIterator();
function OpenDocumentsIterator(openDocs) {
// properties:
	this.openDocs = openDocs;
	this.index = 0;
}

OpenDocumentsIterator.prototype.hasMore = function() {
	return (this.index < this.openDocs.length);
}	

OpenDocumentsIterator.prototype.getNext = function() {
	return (this.index < this.openDocs.length)? this.openDocs[this.index++] : null;
}	

//====================================================================================
// Open Document =====================================================================

// A single open document.
function OpenDocument (path, extension, isNew) {
	this.openDocuments = null; // back reference to open documents container.
	this.isNew = isNew; 
	this.isModifed = false; 
	this.path = path;
	this.extension = extension; // includes leading period.
	this.editorWindow = null; 
	this.selectedNodeIds = [];
	this.selectionListeners = [];
	this.attributeListeners = [];
	this.srcDOM = null;
	this.idMap = null;
	this.lastId = 0;
}
// Build a Map of node ids for the current document.
OpenDocument.prototype.buildIdMap = function() {
	if (!this.srcDOM)
		return;
	this.idMap = new Object();
	var nodes = this.srcDOM.childNodes;
	for (var i=0; i<nodes.length; i++) {
		this.addSrcNodeToIdMap(nodes.item(i));
	}	
}

OpenDocument.prototype.addSrcNodeToIdMap = function(srcNode) {
	var index = "0000" + this.lastId++;
	index = index.substr(-5, 5);
	var id = "X" + index;
	this.idMap[id] = srcNode;
	if (srcNode.hasChildNodes()) {
		var nodes = srcNode.childNodes;
		for (var i=0; i<nodes.length; i++) {
			this.addSrcNodeToIdMap(nodes.item(i));
		}
	}	
}
OpenDocument.prototype.getIdMapNode = function(id) {
	return this.idMap[id];
}
OpenDocument.prototype.getIdMapId = function(srcNode) {
	for (var id in this.idMap) {
		if (this.idMap[id].isSameNode(srcNode))
			return id;
	}
	return null;

}

OpenDocument.prototype.loadDOM = function() {
	//debug("loading:" + this.path);
	var srcDom = null;
	try {
		if (this.path == null) {
			// create new blank document from template.
			srcDOM = Utility.loadDOMFromURI("text/xml", "chrome://xulapp/content/resource/defaultform.xul");
			this.isNew = true;
		}
		else {
			srcDOM = Utility.loadDOMFromURI("text/xml", this.path);
		}
	}
	
	catch (e) { alert(e)};
	return srcDOM;
}

OpenDocument.prototype.clearSelection = function() {
	//debug("clear:" + this.selectedNodeIds.length);
	while (this.selectedNodeIds.length > 0) {
		//debug("Deselect:" + this.selectedNodes[0].localName);
		this.notifyWidgetSelected(this.selectedNodeIds[0], false, false)
	}	
	this.selectedNodeIds = [];
}

OpenDocument.prototype.notifyWidgetSelected = function(nodeId, isSelected, augmentSelection) {
	Private_selectNode(this.selectedNodeIds, nodeId, isSelected);
	// Notify listeners.
	for (var i=0; i< this.selectionListeners.length; i++) {
		var callback = this.selectionListeners[i];
		//debug("OD Notify:" + srcNode.localName + ":" + isSelected);
		try {
			callback(nodeId, isSelected, augmentSelection);
		}
		catch (e) {debug(e);} // do nothing the reference might be dead.
	}
}

function Private_selectNode(selectedNodeIds, nodeId, isSelected) {
	var found = false;
	for (var i=0; i<selectedNodeIds.length; i++) {
		if (found && !isSelected) {
			selectedNodeIds[i - 1] = selectedNodeIds[i];
		}
		if (!found && selectedNodeIds[i] == nodeId) found = true;
	}
	if (found && !isSelected) 
		selectedNodeIds.pop();
	else {
		if (!found && isSelected) selectedNodeIds.push(nodeId);
		// else whoops! already gone.
	}
}

// calback(srcNode, isSelected);
OpenDocument.prototype.registerSelectionListener = function(callback) {
	//debug("Register" + callback);
	this.selectionListeners.push(callback);
}

// The callback parameter should be the original function passed to registerSelectionListener.
OpenDocument.prototype.unregisterSelectionListener = function(callback) {
	//debug("Unregister" + callback);
	var rmv = Utility.removeArrayEntry(this.selectionListeners, callback);	
}

// calback(srcNode, attributeid, oldvalue, newValue, originatingWindow);
// originatingWindow is to enable feedback supression. 
OpenDocument.prototype.registerAttributeListener = function(callback) {
	//debug("Register" + callback);
	this.attributeListeners.push(callback);
}

// The callback parameter should be the original function passed to registerAttributeListener.
OpenDocument.prototype.unregisterAttributeListener = function(callback) {
	//debug("Unregister attribute listener" + callback);
	var rmv = Utility.removeArrayEntry(this.attributeListeners, callback);	
}

OpenDocument.prototype.notifyAttributeChanged = function(nodeId, attributeid, oldValue, newValue, originatingWindow) {
	// Notify listeners.
	//TXN
	this.isModified = true;
	for (var i=0; i< this.attributeListeners.length; i++) {
		var callback = this.attributeListeners[i];
		//debug("OD Notify:" + srcNode.localName + ":" + isSelected);
		try {
			callback(nodeId, attributeid, oldValue, newValue, originatingWindow);
		}
		catch (e) {} // do nothing the reference might be dead.
	}
}
OpenDocument.prototype.saveAs = function() {
	
	var filePicker = Utility.getComponent("@mozilla.org/filepicker;1","nsIFilePicker");
	assert(filePicker,"Unable to obtain file picker.");

	const FP = Components.interfaces.nsIFilePicker;
	filePicker.displayDirectory = new File(xulapp.project.chromePrefix + "content/").parent.nsILocalFile; 
	filePicker.init(window, "Save New Document.", FP.modeSave);
	filePicker.appendFilter(this.extension.substr(1) + " file.", "*" + this.extension);

	var filePicked = filePicker.show();
	if ((filePicked == FP.returnOK || filePicked == FP.returnReplace) && filePicker.file) {
		// get selected file.
		this.path = filePicker.file.path;
		// ensure file extension is applied.
		if (this.path.substr(-this.extension.length) != this.extension)
			this.path += this.extension;
		return this.saveInternal();
	}
	return false;
}

OpenDocument.prototype.save = function() {
	if (this.isNew) {
		return this.saveAs();
	}
	return this.saveInternal();
}

OpenDocument.prototype.saveInternal = function() {
	try {
		var ser = new XULDOMSerializer(xulapp.project.xuloutput);
		ser.serialize(this.srcDOM);
		var file;
		if (ser.DTD) {
			var docType = Utility.getDocType(this.srcDOM); // currently document.doctype is broke
			file = new File(docType.systemId);		
			file.open("w", 0644);	
			file.write(ser.DTD);
			file.close();
		}
		file = new File(this.path);
		file.open("w", 0644);	
		file.write(ser.XML);
		file.close();
		ser = null;
		this.isNew = false;
		this.isModified = false;
	}
	catch (e) {
		alert(e);
	}

}

OpenDocument.prototype.close = function() {
	var openDocs = this.openDocuments; // cache reference as rmvDocument clears it.
	// some cleanup - with GC should not be necessary.
	if (this.editorWindow)
		this.editorWindow.close();
	this.editorWindow = null;
	openDocs.rmvDocument(this);
}

// called from document editor's to determine if document is to be saved - and to save it if required.
OpenDocument.prototype.mightSave = function(promptTitle, promptMsg) {
	if (this.isModified) {
	  	var result = promptService.confirmEx(window, promptTitle, promptMsg,
  						  (promptService.BUTTON_TITLE_SAVE * promptService.BUTTON_POS_0) +
  						  (promptService.BUTTON_TITLE_CANCEL * promptService.BUTTON_POS_1) +
  						  (promptService.BUTTON_TITLE_IS_STRING * promptService.BUTTON_POS_2),
  						  null, null,
  						  xulapp.getMsg("OPENDOC.DontSaveLabel"),
  						  null, {value:0}, result);

  		if (result == 0) 
  			return this.save();
  		else	
			return (result != 1); // not cancel
	}
	else
		return true; // proceed to close.
}


