/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
 
// Open Tools ====================================================================
netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");

// The collection of open tools. This must be a singleton.
function OpenTools(xulapp) {
	this.xulapp = xulapp;
	this.openTools = [];
	this.mainWindow = null;
}
OpenTools.prototype.openTool = function(toolId, fileURI, extension) {
	var openTool = this.findByFileURI(toolId, fileURI);
	if (openTool) { // already open;
		openTool.mainWindow.focus();
		return;	
	}
	openTool = new OpenTool(toolId, fileURI, extension);
	var tool = this.xulapp.tools.getElementById(toolId);
	assert(tool,"tools.xml entry not found for " + toolId + ".");
	openTool.mainWindow = window.openDialog(tool.getAttribute("fileURI"),
													"_blank",
													"chrome,minimizable=yes,dialog=no,resizable=yes,close,titlebar",
													this.xulapp, openTool);	
	this.add(openTool);
}

OpenTools.prototype.add = function(openTool) {
	this.openTools.push(openTool);
	openTool.openTools = this; // Add back reference.
}


OpenTools.prototype.remove = function(openTool) {
	for (var i=0; i<this.openTools.length; i++) {
		if (this.openTools[i] === openTool) {
			openTool.openTools = null; // Remove back reference.			
			this.openTools.splice(i,1);
			break;
		}
	}
}

// Determine if tool is alerady open for this URI.
OpenTools.prototype.findByFileURI = function(toolId, fileURI) {
	var toolsEnum = this.getEnumerator();
	while (toolsEnum.hasMoreElements()) {
		var openTool = toolsEnum.getNext();
		if (openTool.toolId != toolId)
			continue;
		if (openTool.fileURI == fileURI) 
			return openTool;
	}
	return null;
}

OpenTools.prototype.closeAll = function() {
	var stayOpen = false;
	var toolsEnum = this.getEnumerator(true);
	while (toolsEnum.hasMoreElements()) {
		var openTool = toolsEnum.getNext();
		if (openTool.mainWindow.OnClose())
			openTool.mainWindow.close();
	}	
	return (this.length == 0);
}

OpenTools.prototype.__defineGetter__( "length", function() {
	return this.openTools.length;
})

OpenTools.prototype.getEnumerator = function(clone) {
	return new ArrayEnumerator(this.openTools, clone);
}

// Open Program ====================================================================
// (Abstract interface)
// All program tools must supply this interface, or use this version as a prototype.
function OpenTool(toolId, fileURI, extension) {
	//TODO check for opendocument exclusive use.
	this.toolId = toolId;
	this.fileURI = fileURI; // may be null for new file.
	this.extension = extension; // includes leading "."
	this.isNew = (!this.fileURI);
	this.isModified = false;
	this.openTools = null;
	// Default Save prompts:
	this.msgSaveDialogTitle = "OPENTOOL.SaveDialogTitle";
	this.msgSaveDialogMsg = "OPENTOOL.SaveDialogMsg"
	this.msgDontSaveLabel = "OPENTOOL.DontSaveLabel"
}

OpenTool.prototype.saveAs = function() {
	var filePicker = Utility.getComponent("@mozilla.org/filepicker;1","nsIFilePicker");
	assert(filePicker,"Unable to obtain file picker.");

	const FP = Components.interfaces.nsIFilePicker;
	filePicker.displayDirectory = new File(xulapp.project.chromePrefix + "content/").parent.nsILocalFile; 
	filePicker.init(window, "Save New Document.", FP.modeSave);
	filePicker.appendFilter(this.extension.substr(1) + " file.", "*" + this.extension);

	var filePicked = filePicker.show();
	if ((filePicked == FP.returnOK || filePicked == FP.returnReplace) && filePicker.file.path) {
		// get selected file.
		this.fileURI = filePicker.file.path;
		// ensure file extension is applied.
		if (this.fileURI.substr(-this.extension.length) != this.extension)
			this.fileURI += this.extension;
		return this.saveInternal();

	}
	else
		return false;
}

OpenTool.prototype.save = function() {
	if (this.isNew) {
		return this.saveAs();
	}
	return this.saveInternal();
}

OpenTool.prototype.saveInternal = function() {
	try {
		var file = new File(this.fileURI);
		file.open("w", 0644);	
		file.write(this.mainWindow.serialize());
		file.close();
		this.isNew = false;
		this.isModified = false;
		return true;
	}
	catch (e) {
		alert(e);
		return false;
	}
}

// called from program editor's to determine if document is to be saved
// - and to save it if required.
OpenTool.prototype.mightSave = function() {
	if (this.isModified) {
		var promptTitle = xulapp.getMsg(this.msgSaveDialogTitle);
		var promptMsg =  xulapp.getMsg(this.msgSaveDialogMsg, this.fileURI);
		var dontSaveLabel = xulapp.getMsg("OPENTOOL.DontSaveLabel");
	  	var promptService = this.openTools.xulapp.promptService;
  		var result = promptService.confirmEx(this.mainWindow, promptTitle, promptMsg,
  						  (promptService.BUTTON_TITLE_SAVE * promptService.BUTTON_POS_0) +
  						  (promptService.BUTTON_TITLE_CANCEL * promptService.BUTTON_POS_1) +
  						  (promptService.BUTTON_TITLE_IS_STRING * promptService.BUTTON_POS_2),
  						  null, null,
  						  dontSaveLabel,
  						  null, {value:0});

  		if (result == 0) 
  			return this.save();
  		else	
			return (result != 1); // not cancel
	}
	else
		return true; // proceed to close.
}		


OpenTool.prototype.close = function() {
	this.openTools.remove(this);
}
