/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
 
function Project() {
// Properties:
	this.projectName = null;			// Project short name.
	this.projectDisplayName = null;		// Project longer name.
	this.projectAuthor = null;			// Project Author.
	this.projectFileURI = null; 		// URI of project parameters file.
	this.projectDirectoryURI = null;	// URI of project directory: file://c|projectName/
	this.chromePrefix = null;			// chrome URI of project chrome://projectName/
	this.xuloutput = null; 				// Parameters to control the formatting of xul output.
	this.generateDTDEntityFor = null;
}

Project.prototype.openProject = function(projectFileURI) {
	if (!projectFileURI) {
		var filePicker = Utility.getComponent("@mozilla.org/filepicker;1","nsIFilePicker");
		assert(filePicker,"Unable to obtain file picker.");
		if (!filePicker) return false;

		const FP = Components.interfaces.nsIFilePicker;
		filePicker.init(window, "Open XUL App Project", FP.modeOpen);
		filePicker.appendFilter("XUL App Project", "*.xap");

		var filePicked = filePicker.show();
		if (filePicked == FP.returnOK && filePicker.file.path) {
			// load selected project xml file.
			this.projectFileURI = File.pathToURL(filePicker.file.path);
		}
		else
			return false;
	}
	else {
		this.projectFileURI = projectFileURI;
	}
	assert(this.projectFileURI != undefined,"Project file URI is undefined.");
	var projectFile = new File(this.projectFileURI);
	this.projectDirectoryURI = projectFile.parent.URL;
	// XXX do we need to keep dom or re-build/ access when saving project parameters.
	this.DOM = Utility.loadDOMFromURI("text/xml", this.projectFileURI);

	if (this.DOM.documentElement.localName != "project") {
		alert(xulapp.getMsg("PROJECT.NotProjectFile",this.projectFileURI));
		return false;
	}
	this.projectName = projectFile.leaf;	
	this.projectName = this.projectName.substring(0, this.projectName.length - 4);
	this.chromePrefix = "chrome://" + this.projectName + "/";
	this.projectDisplayName = this.DOM.getElementById("projectDisplayName").getAttribute("value");	
	this.projectAuthor = this.DOM.getElementById("projectAuthor").getAttribute("value");	
	this.xuloutput = new Object();
	this.xuloutput.generateDTD = (this.DOM.getElementById("xuloutput.generateDTD").getAttribute("value") == "true");
	this.xuloutput.maxAttributesPerLine = this.DOM.getElementById("xuloutput.maxAttributesPerLine").getAttribute("value");
	this.xuloutput.longAttributeLength = this.DOM.getElementById("xuloutput.longAttributeLength").getAttribute("value");
	this.xuloutput.generateDTDEntityFor = this.DOM.getElementById("xuloutput.createDTDEntityFor").getAttribute("value").split(/\s+/);
	return true;
}
/** Return boolean: Project created ok. **/
Project.prototype.newProject = function(projectName, parentDirectoryName, projectDisplayName, projectAuthor) {
//GOAL: it should be possible to hookup a set of existing files which don't have a chrome url.
//This could arise if a jar file is unpacked for modification.
	try {
		// xml encode
		projectDisplayName = (projectDisplayName)?Utility.xmlEncode(projectDisplayName):projectName;
		projectAuthor = (projectAuthor)?Utility.xmlEncode(projectAuthor):"Unknown";
		
		// !Exists create [projectName] directory.
		var projectDirectory = new File(parentDirectoryName);
		projectDirectory.append(projectName);
		if (!projectDirectory.exists()) {
			projectDirectory.createDir();
		}
		// !Exists create [projectName].xap file.
		var projectXAP = new File(projectDirectory.clone().append(projectName + ".xap").path);
		if (!projectXAP.exists()) {
			// read template.
			var aFile = new File("chrome://xulapp/content/resource/xapTemplate.xml");
			aFile.open("r");
			var contents = aFile.read();
			contents = contents.replace(/%displayName/g, projectDisplayName);
			contents = contents.replace(/%author/g, projectAuthor);
			contents = contents.replace(/(\r\n|\n|\r)/g, Utility.NEWLINE);
			projectXAP.open("w");
			projectXAP.write(contents);
			projectXAP.close();
		}
		this.projectFileURI = projectXAP.URL;

		// !Exists create [projectName].content directory.
		var contentDirectory = projectDirectory.clone().append("content");	
		if (!contentDirectory.exists()) {
			contentDirectory.createDir();
		}
		// generate contents.rdf
		var contentsFile = contentDirectory.clone().append("contents.rdf");
		if (!contentsFile.exists()) {
			// read template.
			aFile = new File("chrome://xulapp/content/resource/contentTemplate.rdf");
			aFile.open("r");
			contents = aFile.read();
			aFile.close();
			contents = contents.replace(/%project/g, projectName);
			contents = contents.replace(/%displayName/g, projectDisplayName);
			contents = contents.replace(/%author/g, projectAuthor);
			contents = contents.replace(/(\r\n|\n|\r)/g, Utility.NEWLINE);
			contentsFile.open("w");
			contentsFile.write(contents);
			contentsFile.close();
		}

		// !Exists create [projectName].locale directory.
		// !Exists create [projectName].locale.[?navigatorcurrentlocale] directory.
		var localeDirectory = projectDirectory.clone().append("locale").append(navigator.language);	
		if (!localeDirectory.exists()) {
			localeDirectory.createDir(); // this will create intermediate locale directory. 
		}
		// generate contents.rdf
		var localeContentsFile = localeDirectory.clone().append("contents.rdf");
		if (!localeContentsFile.exists()) {
			// read template.
			aFile = new File("chrome://xulapp/content/resource/localeTemplate.rdf");
			aFile.open("r");
			contents = aFile.read();
			aFile.close();
			contents = contents.replace(/%project/g, projectName);
			contents = contents.replace(/%author/g, projectAuthor);
			contents = contents.replace(/(\r\n|\n|\r)/g, Utility.NEWLINE);
			localeContentsFile.open("w");
			localeContentsFile.write(contents);
			localeContentsFile.close();
		}

		var newChrome = "";
		// check for existing "content" chrome path.
		var cPath = null;
		try {
			cPath = File.chromeToFileURL("chrome://" + projectName + "/content/");
		}
		catch (e) {}
		if (!cPath) {
			newChrome += "content,install,path," + contentDirectory.path + Utility.DIR_SEPARATOR + Utility.NEWLINE;
		}

		cpath = null;
		try {
			cPath = File.chromeToFileURL("chrome://" + projectName + "/locale/");
		}
		catch (e) {}
		// check for existing "locale" chrome path.
		if (!cPath) {
			newChrome += "locale,install,path," + localeDirectory.path + Utility.DIR_SEPARATOR + Utility.NEWLINE;
		}
		//+++++++++++++++++
		if (newChrome != "" ) {
			// And now for the really scary part.
			// locate installed-chrome.txt and check that exists and is writable.
			var dirServiceProvider = Components.classes["@mozilla.org/file/directory_service;1"]
										 .getService(Components.interfaces.nsIDirectoryServiceProvider);
			var outBoolPersistent = new Object();
			// Get XPCOM Current process directory - this is the Mozilla\bin directory on my machine 
			var installedChrome = dirServiceProvider.getFile("XCurProcD", outBoolPersistent)
			installedChrome = new File(installedChrome.path);
			// Navigate down to installed-chrome.txt.
			installedChrome.append("chrome").append("installed-chrome.txt");
			if (!installedChrome.exists()) {
				alert(xulapp.getMsg("PROJECT.InstalledChromeNotFound",installedChrome.path));
				throw "A fit";
			}
			if (!installedChrome.isWritable()) {
				alert(xulapp.getMsg("PROJECT.InstalledChromeNotWriteable",installedChrome.path));
				throw "A fit";
			}
			// append mode does not work so read file and re-write UGH! 
			installedChrome.open("r");
			var oldChrome = installedChrome.read();
			installedChrome.close();
			installedChrome.open("w");
			installedChrome.write(newChrome + oldChrome);
			installedChrome.close();
			try {
				File_RegistryService.checkForNewChrome(); 
			} 
			catch (e){
				alert(xulapp.getMsg("PROJECT.CantRefreshChrome", navigator.appName));
			} 
		}
		return true;
	}
	catch (e) {
		alert("Create Project: Failure:" + e);
		return false;
	}
}