/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
 
const DATATYPE_DIR = "chrome://xulapp/content/relaxng/datatype/"
const RELAXNG_DATATYPES_URI = 	"http://relaxng.org/ns/datatypes/1.0"

function DatatypeLibraries() {
	this.libraries = new HashMap();
	this.load();
	this.cachedLibraryURI = null; // cached to speed up access.
	this.cachedDatatypeLibrary = null; // cached to speed up access.	
}
DatatypeLibraries.prototype.load = function() {
	var dom = Utility.loadDOMFromURI("text/xml", DATATYPE_DIR + "datatypeLibraries.xml");
	var libs = dom.getElementsByTagName("datatypeLibrary");
	for (var i=0; i < libs.length; i++) {
		var libraryURI = libs.item(i).getAttribute("libraryURI");
		assert(libraryURI,"libraryURI attribute not found");
		var sourceURI = libs.item(i).getAttribute("sourceURI");
		assert(sourceURI,"sourceURI attribute not found");
		 if (sourceURI.substr(0,7) != "chrome:")
		 	sourceURI = DATATYPE_DIR + sourceURI;
		var file = new File(sourceURI);		
		file.open("r");	
		var src = file.read();
		var datatypeLibrary = null;
		eval(src);	// datatypeLibrary is assigned in the eval(src);
		assert(datatypeLibrary,"Datatype libraryURI:" + libraryURI + " sourceURI:" + sourceURI + " did not load correctly.");
		datatypeLibrary.load();
		this.libraries.put(libraryURI, datatypeLibrary);
		var probe=0;
	}

}

DatatypeLibraries.prototype.datatypeAllows = function(libraryURI, type, val, context, params) {
	if (!libraryURI) 
		libraryURI = RELAXNG_DATATYPES_URI;
	if (libraryURI != this.cachedLibraryURI) {	
		this.cachedLibraryURI = libraryURI;
		this.cachedDatatypeLibrary = this.libraries.get(libraryURI);
		assert(this.cachedDatatypeLibrary, "Datatype library:" + libraryURI + " is unknown.");
	}	
	var func = this.cachedDatatypeLibrary.getAllows(type);
	assert(func, "Datatype: " + type + " not supported for datatype library:" + libraryURI + ".");	
	return func(this.cachedDatatypeLibrary, val, context, params);
}
DatatypeLibraries.prototype.datatypeEquals = function(libraryURI, type, val1, context1, val2, context2) {
	if (!libraryURI) 
		libraryURI = RELAXNG_DATATYPES_URI;
	if (libraryURI != this.cachedLibraryURI) {	
		this.cachedLibraryURI = libraryURI;
		this.cachedDatatypeLibrary = this.libraries.get(libraryURI);
		assert(this.cachedDatatypeLibrary, "Datatype library:" + libraryURI + " is unknown.");
	}	
	var func = this.cachedDatatypeLibrary.getEquals(type);
	assert(func, "Datatype: " + type + " not supported by datatype library:" + libraryURI + ".");	
	return func(this.cachedDatatypeLibrary, val1, context1, val2, context2);
}
DatatypeLibraries.prototype.getAllows = function(libraryURI, type) {
	if (!libraryURI)
		libraryURI = RELAXNG_DATATYPES_URI; 
	datatypeLibrary = this.libraries.get(libraryURI);
	assert(datatypeLibrary, "Datatype library:" + libraryURI + " is unknown.");
	var func = datatypeLibrary.getAllows(type);
	assert(func, "Datatype: " + type + " not supported for datatype library:" + libraryURI + ".");	
	return func;
}



var datatypeLibraries = new DatatypeLibraries();
