/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
 
function XMLSchemaDatatypeLibrary() {
	this.allowsMap = new HashMap();
	this.equalsMap = new HashMap();
	this.REinteger  = /^[+-]?\d+$/;;
}
XMLSchemaDatatypeLibrary.prototype.getAllows = function(type) {
	return this.allowsMap.get(type); 
}
XMLSchemaDatatypeLibrary.prototype.getEquals = function(type) {
	return this.equalsMap.get(type); 
}

XMLSchemaDatatypeLibrary.prototype.stringAllows = function(aThis, val, context, params) {
	//alert("XML Schema stringAllows");
	return true;
}
XMLSchemaDatatypeLibrary.prototype.stringEquals = function(aThis, val1, context1, val2, context2) {
	return val1 == val2;
}
XMLSchemaDatatypeLibrary.prototype.anyURIAllows = function(aThis, val, context, params) {
	return true;
}
XMLSchemaDatatypeLibrary.prototype.anyURIEquals = function(aThis, val1, context1, val2, context2) {
	return val1 == val2;
}
XMLSchemaDatatypeLibrary.prototype.IDAllows = function(aThis, val, context, params) {
	return true;
}
XMLSchemaDatatypeLibrary.prototype.IDEquals = function(aThis, val1, context1, val2, context2) {
	return val1 == val2;
}
XMLSchemaDatatypeLibrary.prototype.IDREFAllows = function(aThis, val, context, params) {
	return true;
}
XMLSchemaDatatypeLibrary.prototype.IDREFEquals = function(aThis, val1, context1, val2, context2) {
	return val1 == val2;
}
XMLSchemaDatatypeLibrary.prototype.integerAllows = function(aThis, val, context, params) {
	return aThis.REinteger.test(val);
}
XMLSchemaDatatypeLibrary.prototype.integerEquals = function(aThis, val1, context1, val2, context2) {
	return Number.parseInt(val1) == Number.parseInt(val2);
}
XMLSchemaDatatypeLibrary.prototype.NCNameAllows = function(aThis, val, context, params) {
	return true;
}
XMLSchemaDatatypeLibrary.prototype.NCNameEquals = function(aThis, val1, context1, val2, context2) {
	return val1 == val2;
}
XMLSchemaDatatypeLibrary.prototype.NMTOKENAllows = function(aThis, val, context, params) {
	return true;
}
XMLSchemaDatatypeLibrary.prototype.NMTOKENEquals = function(aThis, val1, context1, val2, context2) {
	return val1 == val2;
}
XMLSchemaDatatypeLibrary.prototype.QNameAllows = function(aThis, val, context, params) {
	return true;
}
XMLSchemaDatatypeLibrary.prototype.QNameEquals = function(aThis, val1, context1, val2, context2) {
	if (val1 == val2)	
		return true;
	//TODO check ns prefixes see if map to same namespace.
	return false;
}
XMLSchemaDatatypeLibrary.prototype.booleanAllows = function(aThis, val, context, params) {
	return (val == "true" || val == "false" || val == "1" || val == "0");
}
XMLSchemaDatatypeLibrary.prototype.booleanEquals = function(aThis, val1, context1, val2, context2) {
	if (val1 == val2)	
		return true;
	if (val1 == "true" && val2 == "1")
		return true;
	if (val1 == "false" && val2 == "0")
		return true;
	return false;
}

XMLSchemaDatatypeLibrary.prototype.load = function() {
	this.allowsMap.put("string", 	this.stringAllows);
	this.equalsMap.put("string", 	this.stringEquals);
	this.allowsMap.put("anyURI", 	this.anyURIAllows);
	this.equalsMap.put("anyURI", 	this.anyURIEquals);
	this.allowsMap.put("ID", 		this.IDAllows);
	this.equalsMap.put("ID", 		this.IDEquals);
	this.allowsMap.put("IDREF", 	this.IDREFAllows);
	this.equalsMap.put("IDREF", 	this.IDREFEquals);
	this.allowsMap.put("integer",	this.integerAllows);
	this.equalsMap.put("integer",	this.integerEquals);
	this.allowsMap.put("NCName", 	this.NCNameAllows);
	this.equalsMap.put("NCName", 	this.NCNameEquals);
	this.allowsMap.put("NMTOKEN", this.NMTOKENAllows);
	this.equalsMap.put("NMTOKEN", this.NMTOKENEquals);
	this.allowsMap.put("QName", 	this.QNameAllows);
	this.equalsMap.put("QName", 	this.QNameEquals);
	this.allowsMap.put("boolean", this.booleanAllows);
	this.equalsMap.put("boolean", this.booleanEquals);
}

datatypeLibrary = new XMLSchemaDatatypeLibrary();
