/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
 
const GRAMMAR = "grammar";
const START = "start";
const REF = "ref";
const INCLUDE = "include";
const DIV = "div";
const GROUP = "group";
const DEFINE = "define";
const ELEMENT = "element";
const ATTRIBUTE = "attribute";
const ONEORMORE = "oneOrMore";
const OPTIONAL = "optional";
const ZEROORMORE = "zeroOrMore";
const CHOICE = "choice";
const INTERLEAVE = "interleave";
const MIXED = "mixed";
const VALUE = "value";
const DATA = "data";
const TEXT = "text";
const LIST = "list";
const EMPTY = "empty";
const NOTALLOWED = "notAllowed";
const NAME = "name";
const ANYNAME = "anyName";
const NSNAME = "nsName";
const EXCEPT = "except";
const PARAM = "param";
const UNKNOWN = "unknown";
const SEQUENCE = "group";
const EMPTYSEQUENCE = "empty-sequence";
const EMPTYCHOICE = "empty-choice";
var RE_Whitespace = /^\s*$/;
var RE_SplitWhitespace = /\s/;
var RE_TrimWhitespace = /^\s*(.+)\s*$/;

function Pattern() {
	this.patName = null;
	this.isNullable = false;
}
Pattern.prototype.attributesToString = function() {
	return "";
}

/** This function can be called with a varying number of parameters each specifying the attribute to format.
	Undefined attributes are not reported.
**/
Pattern.prototype.attr = function() {
	var val = "";
	// add in common attributes.
	for (var i=0; i<arguments.length; i++) {
		var attrName = arguments[i];


		if (attrName in this) {
			var attrObj = this[attrName];
			if (attrObj != null) {
				if (!(attrObj instanceof Pattern)) { // if name is pattern defer to element content.
					if (val != "")
						val += " ";
					val += attrName + '="' + attrObj + '"';
				}	
			}	
		}
	}
	//TODO remove this if no longer needed.	
	if ("patternSet" in this) {
		if (val != "")
			val += " ";
		val += 'patternSet="';
		var keys = this.patternSet.keys();
		for (var j=0; j < keys.length; j++) {
			if (j > 0)
				val += " ";
			var p = patternSet.get(keys[j]);
			val += p.name;
		}
		val += '"';
	}
	return val;
}
/** The parent pattern containing the required pattern is returned. **/
Pattern.prototype.getElementsByTagName = function(name) {
	var elements = [];
	this.getElementsByTagNameInternal(name, elements, null);
	return elements;	
}

Pattern.prototype.getElementsByTagNameInternal = function(name, elements, parentPattern) {
	if (name == "*" || this.patName == name) 
		elements[elements.length] = parentPattern;
	if (this instanceof UnaryPattern) {
		if (this.p1)
			this.p1.getElementsByTagNameInternal(name, elements, this);
	}
	else 
		if (this instanceof BinaryPattern) {
			if (this.p1)
				this.p1.getElementsByTagNameInternal(name, elements, this);
			if (this.p2)
				this.p2.getElementsByTagNameInternal(name, elements, this);
		}
}

Pattern.prototype.containsChoice = function(p) {
    return (this == p);
}
Pattern.prototype.deriv = function(theNode, level, errorList) {
	//logger.log ("<" + theNode.nodeName + "> " + this.patName + " (Pattern)", level);
	return EmptyChoice;
}
Pattern.prototype.attributesToString = function() {
	return this.attr('ns', "datatypeLibarary");
}
Pattern.prototype.buildElementContent = function(theElem) {
}

function UnaryPattern() {
	this.p1 = null;
}
UnaryPattern.prototype = new Pattern;
UnaryPattern.prototype.remove = function() {
	this.p1 = null;
}
UnaryPattern.prototype.buildElementContent = function(theElem) {
	if (this.p1)
		this.p1.buildElementContent(theElem);
}

function BinaryPattern() {
	this.p1 = null;
	this.p2 = null;
}
BinaryPattern.prototype = new Pattern;
BinaryPattern.prototype.buildElementContent = function(theElem) {
	if (this.p1)
		this.p1.buildElementContent(theElem);
	if (this.p2)
		this.p2.buildElementContent(theElem);
}

//=============================================================================================================
function GrammarPattern() {
	this.patName = GRAMMAR; 
	this.parentGrammar = null;
	this.defineMap = new HashMap();
	this.start = null;
	this.visitID = 0;
	this.baseURI = null;
}
GrammarPattern.prototype = new UnaryPattern;
//=============================================================================================================
function StartPattern(grammar) {
	this.patName = START;
	this.test = "fred";
	grammar.start = this;
}
StartPattern.prototype = new UnaryPattern;
StartPattern.prototype.deriv = function(theNode, level, errorList) {
	//logger.log ("<" + theNode.nodeName + "> " + this.patName, level);
	return this.p1.deriv(theNode, level, errorList);
}
//=============================================================================================================
function RefPattern(name, grammar) {
	this.patName = REF;
	this.grammar = grammar;
	this.name = name;
	this.refDefine = null;
}
RefPattern.prototype = new UnaryPattern;
RefPattern.prototype.attributesToString = function() {
	return this.attr('name');
}
RefPattern.prototype.deriv = function(theNode, level, errorList) {
	//logger.log ("<" + theNode.nodeName + "> " + this.patName + " name=" + this.name, level);
	return this.refDefine.p1.deriv(theNode, level, errorList);
}
RefPattern.prototype.buildElementContent = function(theElem) {
	return this.refDefine.p1.buildElementContent(theElem);
}
//=============================================================================================================
function IncludePattern(href) {
	this.patName = INCLUDE;
	this.href = href;
	this.iGrammar = null; // temporary reference to included grammer;
}
IncludePattern.prototype = new UnaryPattern;
IncludePattern.prototype.attributesToString = function() {
	return this.attr('href');
}
//=============================================================================================================
function DivPattern(p1) {
	this.patName = DIV;
	if (p1)
		this.p1 = p1;
}
DivPattern.prototype = new UnaryPattern;
//=============================================================================================================
function DefinePattern(name, combine, childPattern) {
	this.patName = DEFINE;
	this.name = name;
	if (combine)
		this.combine = combine;
	this.p1 = childPattern;
	this.isNullable = childPattern.isNullable;
	this.visitID = -1; // Tag to stop recursive refs getting out of hand.	
	this.baseURI = null;
	this.referenced = false;
}
DefinePattern.prototype = new UnaryPattern;
DefinePattern.prototype.attributesToString = function() {
	return this.attr('name', 'combine');
}
DefinePattern.prototype.deriv = function(theNode, level, errorList) {
	//logger.log ("<" + theNode.nodeName + "> " + this.patName + " name=" + this.name, level);
	return this.p1.deriv(theNode, level, errorList);
}
//=============================================================================================================
function ElementPattern(name, ns) {
	this.patName = ELEMENT;
	this.name = name;
	this.ns = ns;
}
ElementPattern.prototype = new UnaryPattern;
ElementPattern.prototype.attributesToString = function() {
	return this.attr('name');
}
ElementPattern.prototype.deriv = function(theNode, level, errorList) {
	//logger.log ("<" + theNode.nodeName + "> " + this.patName + " name=" + this.name, level);
	if (!theNode)
		return EmptyChoice;
	
	if (!this.matchNamespace(theNode))
		return EmptyChoice;

	//errorList.log(theNode, "@@ELEM:" + theNode.localName + " " +  theNode.getAttribute("name") + " Begin ", level);
		
	// Validate attributes:	
	var p = this.p1;
	var d = p;
	var attributes = theNode.attributes;
	for (var i=0; i<attributes.length; i++) {
		var attr = attributes.item(i);
		if (attr.nodeName != "xmlns") {
			d = d.deriv(attr, level + 1, errorList);
			if (d==EmptyChoice)
				errorList.log(attr, "  @@ATTR:" + attr.localName + " Invalid");
			//var out = serializer.serialize(d);
			//logger.log(out.xml, level + 1);
		}	
	}

	// Validate children
	var childNode = theNode.firstChild
	//var d = p;
	while (childNode) {
		if (childNode.nodeType == NODE_TYPE_COMMENT) {
			childNode = childNode.nextSibling;
			continue; 
		}
		if (childNode.nodeType == NODE_TYPE_TEXT || childNode.nodeType == NODE_TYPE_CDATA) {
			var isWS = RE_Whitespace.test(childNode.nodeValue);
			if (!isWS) {
				d = d.deriv(childNode, level + 1, errorList);
				if (d == EmptyChoice) {
					errorList.log(childNode, "  @@TEXT:" + childNode.nodeValue + " Invalid");
				}	
				
			}	
			childNode = childNode.nextSibling;
			continue; 
		}
		d = d.deriv(childNode, level + 1, errorList);
		if (d == EmptyChoice) {
			errorList.log(childNode, "  @@CHILD ELEM:" + childNode.localName + " Invalid");
		}
		childNode = childNode.nextSibling;
	}
	//errorList.log(theNode, "@@ELEM:" + theNode.localName + " " +  theNode.getAttribute("name") +  " End: nullable=" + d.isNullable, level);
	return (d.isNullable)? EmptySequence : EmptyChoice;
}
ElementPattern.prototype.matchNamespace = function(theNode) {
	if (theNode.nodeType != NODE_TYPE_ELEMENT)
		return false;
	// a namespace name.	
	if (this.name instanceof Pattern)
		return this.name.matchNamespace(theNode);
	// a simple name.
	//alert("mns:" + theNode.tagName + "  " + this.name);
	if (theNode.localName != this.name)
		return false;
	if (!theNode.namespaceURI)
		return (this.ns == "");
	return (theNode.namespaceURI == this.ns);
}
ElementPattern.prototype.buildElementContent = function(theElem) {
	if (theElem)
		theElem.elemList.put(this.name, this);
	this.elemList = new HashMap();
	this.attrList = new HashMap();
	if (this.p1)
		this.p1.buildElementContent(theElem);
}
//=============================================================================================================
function AttributePattern(name, ns) {
	this.patName = ATTRIBUTE;
	this.name = name;
	this.ns = ns;
}
AttributePattern.prototype = new UnaryPattern;
AttributePattern.prototype.attributesToString = function() {
	return this.attr('name');
}
AttributePattern.prototype.deriv = function(theNode, level, errorList) {
	//logger.log ("<" + theNode.nodeName + "> " + this.patName + " name=" + this.name, level);
	if (!theNode)
		return EmptyChoice;
	if (!this.matchNamespace(theNode))
		return EmptyChoice;
	if (this.p1)	
		return this.p1.deriv(theNode,level, errorList)		
	return EmptySequence;
}
AttributePattern.prototype.matchNamespace = function(theNode) {
	if (theNode.nodeType != NODE_TYPE_ATTRIBUTE)
		return false;
	// a namespace name.	
	if (this.name instanceof Pattern)
		return this.name.matchNamespace(theNode);
	// a simple name.
	if (theNode.localName != this.name)
		return false;
	if (!theNode.namespaceURI)
		return (this.ns == "");
	return (theNode.namespaceURI == this.ns);
}
AttributePattern.prototype.buildElementContent = function(theElem) {
	if (theElem)
		theElem.attrList.put(this.name, this);
	// no need to go any lower.
}
//=============================================================================================================
// This pattern is only used for serializing output.
function OptionalPattern(p1) {
	this.patName = OPTIONAL;
	this.p1 = p1;
}
OptionalPattern.prototype = new UnaryPattern;
//=============================================================================================================
function OneOrMorePattern(p1) {
	this.patName = ONEORMORE;
	this.p1 = p1;
}
OneOrMorePattern.prototype = new UnaryPattern;
OneOrMorePattern.prototype.deriv = function(theNode, level, errorList) {
	//logger.log ("<" + theNode.nodeName + "> " + this.patName, level);
	var d1 = this.p1.deriv(theNode,level, errorList);
	return this.makeSequence(	d1, 
								this.makeChoice(this.makeOneOrMore(this.p1, level), EmptySequence, level), level );
}
//=============================================================================================================
function ChoicePattern(p1, p2) {
	this.patName = CHOICE;
	this.p1 = p1;
	this.p2 = p2;
	this.isNullable = p1.isNullable || p2.isNullable;
}
ChoicePattern.prototype = new BinaryPattern;
ChoicePattern.prototype.attributesToString = function() {
	return this.attr();
}
ChoicePattern.prototype.remove = function() {
	if (this.p2 == EmptyChoice)
		this.p1 = EmptyChoice;
	else {
		this.p1 = this.p2.p1;
		this.p2 = this.p2.p2;
	}	
}
ChoicePattern.prototype.containsChoice = function(p) {
    return ( this.p1.containsChoice(p) || this.p2.containsChoice(p) );
}
ChoicePattern.prototype.deriv = function(theNode, level, errorList) {
	//logger.log ("<" + theNode.nodeName + "> " + this.patName, level);
	var d1 = this.p1.deriv(theNode, level, errorList);
	var d2 = this.p2.deriv(theNode, level, errorList);
	return this.makeChoice(d1, d2, level);
}
ChoicePattern.prototype.matchNamespace = function(theNode) {
	return (this.p1.matchNamespace(theNode) || this.p2.matchNamespace(theNode));
}
//=============================================================================================================
function SequencePattern(p1, p2) {
	this.patName = SEQUENCE;
	this.p1 = p1;
	this.p2 = p2;
	this.isNullable = p1.isNullable && p2.isNullable;
}
SequencePattern.prototype = new BinaryPattern;
SequencePattern.prototype.attributesToString = function() {
	return this.attr();
}
SequencePattern.prototype.remove = function() {
	if (this.p2 == EmptySequence)
		this.p1 = EmptySequence;
	else {
		this.p1 = this.p2.p1;
		this.p2 = this,p2.p2;
	}		
}
SequencePattern.prototype.deriv = function(theNode, level, errorList) {
	//logger.log ("<" + theNode.nodeName + "> " + this.patName, level);
	if (!theNode)
		return EmptyChoice;
	var d1, d2;
	if (theNode.nodeType == NODE_TYPE_ATTRIBUTE) {
		d1 = this.p1.deriv(theNode, level, errorList);
		d2 = this.p2.deriv(theNode, level, errorList);
		return this.makeChoice(
			this.makeInterleave( d1,	this.p2, level),
			this.makeInterleave( this.p1, d2, level), level);
	}
	d1 = this.p1.deriv(theNode, level, errorList);
	var tem = this.makeSequence(	d1, this.p2,level);
	if (this.p1.isNullable) {
		d2 = this.p2.deriv(theNode, level, errorList);
		return this.makeChoice(tem, d2, level);
	}	
	else	
		return tem;
}
// This function is called only within Except Patterns.
SequencePattern.prototype.matchNamespace = function(theNode) {
	return ( this.p1.matchNamespace(theNode) || this.p2.matchNamespace(theNode) );
}
//=============================================================================================================
function InterleavePattern(p1, p2) {
	this.patName = INTERLEAVE;
	this.p1 = p1;
	this.p2 = p2;
	this.isNullable = p1.isNullable && p2.isNullable;
}
InterleavePattern.prototype = new BinaryPattern;
InterleavePattern.prototype.remove = function() {
	if (this.p2 == EmptySequence)
		this.p1 = EmptySequence;
	else {
		this.p1 = this.p2.p1;
		this.p2 = this,p2.p2;
	}		
}
InterleavePattern.prototype.deriv = function(theNode, level, errorList) {
	//logger.log ("<" + theNode.nodeName + "> " + this.patName, level);
	var d1 = this.p1.deriv(theNode, level, errorList);
	var d2 = this.p2.deriv(theNode, level, errorList);
	return this.makeChoice(
		this.makeInterleave(d1, this.p2, level),
		this.makeInterleave(this.p1, d2, level), level);
}
//=============================================================================================================
function ValuePattern(type, datatypeLibrary) {
	this.patName = VALUE;
	this.type = type;
	this.datatypeLibrary = datatypeLibrary;
	this.value = null;
}
ValuePattern.prototype = new Pattern;
ValuePattern.prototype.deriv = function(theNode, level, errorList) {
	if (!theNode)
		return EmptyChoice;

	if (theNode.nodeType == NODE_TYPE_ATTRIBUTE || theNode.nodeType == NODE_TYPE_TEXT || theNode.nodeType == NODE_TYPE_CDATA) {
		if (datatypeLibraries.datatypeEquals(	this.datatypeLibrary,
															(this.type || "string"),
															theNode.nodeValue,
															null,
															this.value,
															null) )
			return EmptySequence;
		else
			return EmptyChoice;
	}
	return EmptyChoice;
}
ValuePattern.prototype.attributesToString = function() {
	return this.attr('type', 'datatypeLibrary');
}
//=============================================================================================================
function DataPattern(type, datatypeLibrary) {
	this.patName = DATA;
	this.type = type;
	this.datatypeLibrary = datatypeLibrary;
}
DataPattern.prototype = new UnaryPattern;
DataPattern.prototype.deriv = function(theNode, level, errorList) {
	if (!theNode)
		return EmptyChoice;

	if (theNode.nodeType == NODE_TYPE_ATTRIBUTE || theNode.nodeType == NODE_TYPE_TEXT || theNode.nodeType == NODE_TYPE_CDATA) {
		if (datatypeLibraries.datatypeAllows(	this.datatypeLibrary,
															(this.type || "string"),
															theNode.nodeValue,
															null) ) //TODO pass param's
			return EmptySequence;
		else
			return EmptyChoice;
	}
	return EmptyChoice;
}
DataPattern.prototype.attributesToString = function() {
	return this.attr('type', 'datatypeLibrary');
}
//=============================================================================================================
function ParamPattern() {
	this.patName = PARAM;
	this.value = null;
}
ParamPattern.prototype = new Pattern;
//=============================================================================================================
function TextPattern() {
	this.patName = TEXT;
	this.isNullable = true;
}
TextPattern.prototype = new Pattern;
TextPattern.prototype.deriv = function(theNode, level, errorList) {
	if (theNode.nodeType == NODE_TYPE_TEXT || theNode.nodeType == NODE_TYPE_CDATA) 
		return Text;
	else
		return EmptyChoice;
}
//=============================================================================================================
function ListPattern(separatorRegExp) {
	this.patName = LIST;
	if (!separatorRegExp)
		this.separatorRegExp = RE_SplitWhitespace;
	else	
		this.separatorRegExp = new RegExp(separatorRegExp);
}
ListPattern.prototype = new UnaryPattern;
ListPattern.prototype.deriv = function(theNode, level, errorList) {
	if (theNode.nodeType == NODE_TYPE_ATTRIBUTE || theNode.nodeType == NODE_TYPE_TEXT || theNode.nodeType == NODE_TYPE_CDATA) {
		try {
			// In 2002 XUL cloneNode and createAttribute are not implemented for attributes
			// They figured it would not be needed (sigh).
			// Risky - we modify the node value and then restore it!
			var tempAttrValue = theNode.nodeValue;
			var vals = theNode.nodeValue.split(this.separatorRegExp);
			var d = this.p1;
			for (var i=0; i<vals.length; i++) {
				theNode.nodeValue = vals[i];
				d = d.deriv(theNode, level, errorList);
			}
		}
		finally {
			theNode.NodeValue = tempAttrValue;
		}
		return (d == EmptyChoice) ?EmptyChoice :EmptySequence;
	}
	return EmptyChoice;
}
//=============================================================================================================
function NamePattern(name, ns) {
	this.patName = NAME;
	this.name = name;
	this.ns = ns;
}
NamePattern.prototype = new Pattern;
NamePattern.prototype.matchNamespace = function(theNode) {
	if (theNode.localName != this.name || (theNode.namespaceURI||"") != this.ns)
		return false;
	if (this.p1 == EmptySequence)
		return true;
	return this.p1.matchNamespace(theNode);	
}
//=============================================================================================================
function AnyNamePattern() {
	this.patName = ANYNAME;
}
AnyNamePattern.prototype = new UnaryPattern;
AnyNamePattern.prototype.matchNamespace = function(theNode) {
	if (this.p1 == EmptyChoice)
		return true;
	return this.p1.matchNamespace(theNode);	
}
//=============================================================================================================
function NSNamePattern() {
	this.patName = NSNAME;
}
NSNamePattern.prototype = new UnaryPattern;
NSNamePattern.prototype.matchNamespace = function(theNode) {
	if ((theNode.namespaceURI||"") != this.ns)
		return false;
	if (this.p1 == EmptyChoice)
		return true;
	return this.p1.matchNamespace(theNode);	
}
//=============================================================================================================
function ExceptPattern(p1) {
	this.patName = EXCEPT;
	this.p1 = p1;
}
ExceptPattern.prototype = new BinaryPattern;
ExceptPattern.prototype.matchNamespace = function(theNode) {
	return (!this.p1.matchNamespace(theNode));	
}
//=============================================================================================================
function UnknownPattern() {
	this.patName = UNKNOWN;
}
UnknownPattern.prototype = new UnaryPattern;
//=============================================================================================================
function EmptySequencePattern() {
	this.patName = EMPTYSEQUENCE;
	this.isNullable = true;
}
EmptySequencePattern.prototype = new Pattern;
EmptySequencePattern.prototype.deriv = function(theNode, level, errorList) {
	//logger.log ("<" + theNode.nodeName + "> " + this.patName, level);
	return EmptyChoice;
}
//=============================================================================================================
function EmptyChoicePattern() {
	this.patName = EMPTYCHOICE;
	this.isNullable = false;
}
EmptyChoicePattern.prototype = new Pattern;
//=============================================================================================================
// Static patterns referenced from many other patterns.
var Text = new TextPattern();
const EmptySequence = new EmptySequencePattern();
const EmptyChoice = new EmptyChoicePattern();
const DefaultAttributePattern = new DataPattern("string",""); // Used for <attribute name="xyz"/> usage.
// Builders==================================================================================================
Pattern.prototype.makeSequence = function(p1, p2, level) {
	//logger.log("makeSequence:" + p1.patName + ", " + p2.patName, level);
    if (p1 == EmptySequence)
      return p2;
    if (p2 == EmptySequence)
      return p1;
    if (p1 == EmptyChoice || p2 == EmptyChoice)
      return EmptyChoice;
    if (p1 instanceof SequencePattern) {
      return this.makeSequence(p1.p1, this.makeSequence(p1.p2, p2, level), level);
    }
    return new SequencePattern(p1, p2); // intern?
}

Pattern.prototype.makeChoice = function(p1, p2, level) {
	//logger.log("makeChoice:" + p1.patName + ", " + p2.patName, level);
    if (p1 == EmptyChoice)
    	return p2;
    if (p2 == EmptyChoice)
    	return p1;
    if (p1 == EmptySequence) {
    	if (p2.isNullable)
			return p2;
      	// Canonicalize position of emptyChoice.
      	return this.makeChoice(p2, p1, level);
    }
    if (p2 == EmptySequence && p1.isNullable)
    	return p1;
    if (p1 instanceof ChoicePattern) {
      return this.makeChoice(p1.p1, this.makeChoice(p1.p2, p2, level), level);
    }
    // elimiate cases like choice(p1 choice(p1, p2))
    if (p2.containsChoice(p1))
    	return p2;
    return new ChoicePattern(p1, p2); // intern ??
}
function convertToChoice(p1, p2) {
	//logger.log("convertToChoice:" + p1.patName + ", " + p2.patName);
    if (p1 == EmptyChoice)
    	return p2;
    if (p2 == EmptyChoice)
    	return p1;
    if (p1 == EmptySequence) {
    	if (p2.isNullable)
			return p2;
      	// Canonicalize position of emptyChoice.
      	return convertToChoice(p2, p1);
    }
    if (p2 == EmptySequence && p1.isNullable)
    	return p1;
    if (p1 instanceof ChoicePattern) {
      return convertToChoice(p1.p1, convertToChoice(p1.p2, p2));
    }
    if (p1 instanceof SequencePattern) {
      return convertToChoice(p1.p1, convertToChoice(p1.p2, p2));
    }
    if (p2 instanceof SequencePattern) {
      return convertToChoice(p2.p1, convertToChoice(p1, p2.p2));
    }
    // elimiate cases like choice(p1 choice(p1, p2))
    if (p2.containsChoice(p1))
    	return p2;
    return new ChoicePattern(p1, p2); // intern ??
}

Pattern.prototype.makeInterleave = function(p1, p2, level) {
	//logger.log("makeInterleave:" + p1.patName + ", " + p2.patName, level);
    if (p1 == EmptySequence)
      return p2;
    if (p2 == EmptySequence)
      return p1;
    if (p1 == EmptyChoice || p2 == EmptyChoice)
      return EmptyChoice;
    if (p1 instanceof InterleavePattern) {
      return this.makeInterleave(p1.p1, this.makeInterleave(p1.p2, p2, level), level);
    }
    return new InterleavePattern(p1, p2); // intern?
}

function convertToInterleave(p1, p2) {
	//logger.log("makeInterleave:" + p1.patName + ", " + p2.patName);
    if (p1 == EmptySequence)
      return p2;
    if (p2 == EmptySequence)
      return p1;
    if (p1 == EmptyChoice || p2 == EmptyChoice)
      return EmptyChoice;
    if (p1 instanceof SequencePattern) {
      return convertToInterleave(p1.p1, convertToInterleave(p1.p2, p2));
    }
    if (p2 instanceof SequencePattern) {
      return convertToInterleave(p2.p1, convertToInterleave(p1, p2.p2));
    }
      
    if (p1 instanceof InterleavePattern) {
      return convertToInterleave(p1.p1, convertToInterleave(p1.p2, p2));
    }
    return new InterleavePattern(p1, p2); // intern?
}

Pattern.prototype.makeValue = function(datatype, obj) {
    return new ValuePattern(datatype, obj);  // intern ?
}

Pattern.prototype.makeData = function(datatype) {
    return new DataPattern(datatype); // intern ?
}

Pattern.prototype.makeDataExcept = function(datatype, except) {
    return new DataExceptPattern(datatype, except); // intern ?
}

Pattern.prototype.makeOneOrMore = function(p1, level) {
	//logger.log("makeOneOrMore:" + p1.patName + " " + p1.name, level);
    if (p1 == Text
	|| p1 == EmptySequence
	|| p1 == EmptyChoice
	|| p1 instanceof OneOrMorePattern)
      return p1;
    return new OneOrMorePattern(p1); // intern ?
}

Pattern.prototype.makeList = function(p1) {
    if (p1 == EmptyChoice)
      return p1;
    return new ListPattern(p1); // intern ?
  }

Pattern.prototype.makeElement = function(nameClass, p1) {
    return new ElementPattern(nameClass, p1); // intern ?
}

Pattern.prototype.makeAttribute = function(nameClass, p1) {
    if (p1 == EmptyChoice)
    	return p1;
    return new AttributePattern(nameClass, p1); // intern ?
}
