/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
 
function Schemas() {
	this.schemas = {};
	this.schemaDefs = Utility.loadDOMFromURI("text/xml", "chrome://xulapp/content/resource/schemas.xml");
	this.XPATH = Components.classes["@mozilla.org/dom/xpath-evaluator;1"].createInstance(Components.interfaces.nsIDOMXPathEvaluator);
	this.NSResolver = this.XPATH.createNSResolver(this.schemaDefs.documentElement);
	this.ORDERED_NODE_ITERATOR_TYPE = 5; // Ordered.
}
Schemas.prototype.getSchemaByNamespaceURI = function(namespaceURI) {
	if (!namespaceURI)
		return null;
	if (namespaceURI in this.schemas)
		return this.schemas[namespaceURI];
	var path = "//schema[@namespaceURI='" + namespaceURI + "']";
 	var nodeIterator = this.XPATH.evaluate(path, this.schemaDefs, this.NSResolver, this.ORDERED_NODE_ITERATOR_TYPE, null);	
	var schemaDef = nodeIterator.iterateNext();
	if (!schemaDef)
		return null;
	return this.loadSchema(schemaDef);	
}
Schemas.prototype.getSchemaByPublicId = function(publicId) {
	var path = "//schema[@publicId='" + publicId + "']";
 	var nodeIterator = this.XPATH.evaluate(path, this.schemaDefs, this.NSResolver, this.ORDERED_NODE_ITERATOR_TYPE, null);	
	var schemaDef = nodeIterator.iterateNext();
	if (!schemaDef)
		return null;
	var fileURI = schemaDef.getAttribute("fileURI");	
	return this.getSchemaByFileURI(fileURI);
}
Schemas.prototype.getSchemaByFileURI = function(fileURI) {
	var path = "//schema[@fileURI='" + fileURI + "']";
 	var nodeIterator = this.XPATH.evaluate(path, this.schemaDefs, this.NSResolver, this.ORDERED_NODE_ITERATOR_TYPE, null);	
	var schemaDef = nodeIterator.iterateNext();
	if (!schemaDef)
		return null;
	return this.loadSchema(schemaDef);	
}

Schemas.prototype.loadSchema = function(schemaDef) {
	var fileURI = schemaDef.getAttribute("fileURI");	
	if (!fileURI)
		return null;
	var schema = null;
	var loader = new RNGSchemaLoader(fileURI);
	var grammar = loader.load();
	if (grammar) {
		var id = schemaDef.getAttribute("id");
		var description = schemaDef.getAttribute("description");
		var namespaceURI = schemaDef.getAttribute("namespaceURI");	
		schema =  new Schema(grammar, description, id);
		this.schemas[namespaceURI] = schema;
	}
	return schema;
}
Schemas.prototype.getRelaxNGPrototypes = function() {
	protos = {};
	protos.Pattern		= Pattern.prototype;
	protos.Start 		= StartPattern.prototype;
	protos.Ref			= RefPattern.prototype;
	protos.Define		= DefinePattern.prototype;
	protos.Element		= ElementPattern.prototype;
	protos.Attribute	= AttributePattern.prototype;
	protos.OneOrMore	= OneOrMorePattern.prototype;
	protos.Choice		= ChoicePattern.prototype;
	protos.Sequence	= SequencePattern.prototype;
	protos.Interleave	= InterleavePattern.prototype;
	protos.Value		= ValuePattern.prototype;
	protos.Data			= DataPattern.prototype;
	protos.Text			= TextPattern.prototype;
	protos.List			= ListPattern.prototype;
	protos.EmptySequence	= EmptySequencePattern.prototype;
	protos.Name			= NamePattern.prototype;
	protos.AnyName		= AnyNamePattern.prototype;
	protos.NSName		= NSNamePattern.prototype;
	protos.Except		= ExceptPattern.prototype;
	protos.EmptySequenceObj = EmptySequence;
	protos.EmptyChoiceObj = EmptyChoice;
	protos.TextObj = Text;
	protos.PatternObj = Pattern;
	protos.datatypeLibraries = datatypeLibraries;
	return protos;
}

function Schema(grammar, description, id) {
	this.grammar = grammar;
	this.description = description;
	this.id = id;
}

Schema.prototype.validateDocument = function(dom, errorList) {
	//TODO PI's
	return (EmptySequence == this.grammar.start.deriv(dom.documentElement, null, errorList));
	
}
