/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
 
const XULAPP_EXTENSIONS = "http://xulapp.org/relaxng/extensions/1.0";
var QName = { name:null, ns:"" }
var PATTERN = new Pattern();
function RNGSchemaLoader(uri) {
	nl = Utility.NEWLINE;
	this.uri = uri;
	this.dom = null;
	this.includeMap = null;
}

RNGSchemaLoader.prototype.load = function() {
	this.includeMap = new HashMap();
	var g = this.loadGrammar(this.uri);
	this.normalize(g);
	delete this.includeMap; // nolonger used.
	return g;
}

RNGSchemaLoader.prototype.loadGrammar = function(uri) {
	if (this.includeMap.exists(uri)) {
		assert(false,"Recursive include" + uri);
	}
	this.includeMap.put(uri); // PUSH
	
	this.dom = Utility.loadDOMFromURI("text/xml", uri);
	var documentElement = this.dom.documentElement;
	documentElement.relaxBaseURI = this.extractBaseURI(uri); // temp work variable for this grammar.
	assert((documentElement.localName == "grammar"), "grammar element missing for:" + this.uri);
	var g = this.buildPattern(null, "", documentElement, GRAMMAR);
	this.includeMap.remove(uri); // POP
	return g;
}

RNGSchemaLoader.prototype.buildPattern = function(grammar, define, theNode, context) {
	if (!theNode)
		return EmptyChoice;
	var childNode = this.nextElement(theNode.firstChild);
	var p;
	switch (theNode.localName) {
	case REF:
		return new RefPattern(theNode.getAttribute("name"), grammar);
	case INCLUDE:
		p = new IncludePattern(theNode.getAttribute("href"));
		p.iGrammar = this.loadGrammar(theNode.ownerDocument.documentElement.relaxBaseURI + p.href);
		p.p1 = this.buildSeq(grammar, define, childNode, INCLUDE);
		p = this.mergeInclude(grammar, p);
		return EmptySequence;
	case DIV:
		p = this.buildSeq(grammar, define, childNode, context);
		return p;
	case GROUP:
		return PATTERN.makeSequence(this.buildSeq(grammar, define, childNode, context) , EmptySequence);	
	case DEFINE:
		var children = this.buildSeq(grammar, define, childNode, context); 
		p = new DefinePattern(theNode.getAttribute("name"), theNode.getAttribute("combine"), children);
		if (context != INCLUDE) {
			if (grammar.defineMap.exists(p.name)) {
				this.combineDefine(grammar, p)
			}
			else {
				grammar.defineMap.put(p.name, p);
			}
			return EmptySequence; // defines are not linked.
		}
		return p;

		
	case ELEMENT:
		this.getQName(theNode);
		p = new ElementPattern(QName.name, QName.ns);
		if (!p.name) {
			if (childNode) {
				assert( (childNode.localName == NAME || childNode.localName == ANYNAME || childNode.localName == NSNAME),
					"No name defined for Element " + theNode.localName);
				p.name = this.buildPattern(grammar, define, childNode, context);
				if (childNode)
					childNode = this.nextElement(childNode.nextSibling);
			}
			else
				assert( false, "No name defined for Element " + theNode.localName);
		}
		p.p1 = this.buildSeq(grammar, define, childNode, context);
		return p;
	case ATTRIBUTE:
		this.getQName(theNode);
		p = new AttributePattern(QName.name, QName.ns);
		if (!p.name) {
			if (childNode) {
				assert( (childNode.localName == NAME || childNode.localName == ANYNAME || childNode.localName == NSNAME),
					"No name defined for Attribute " + theNode.localName);
				p.name = this.buildPattern(grammar, define, childNode, context);
				if (childNode)
					childNode = this.nextElement(childNode.nextSibling);
				else
					assert( false, "No name defined for Attribute " + theNode.localName);
			}	
		}
		p.p1 = this.buildPattern(grammar, define, childNode, context);
		if (p.p1 == EmptyChoice)
			p.p1 = DefaultAttributePattern; // default type="string" and ns=""
		return p;
	case ONEORMORE:
		p = new OneOrMorePattern();
		p.p1 = this.buildSeq(grammar, define, childNode, context);
		return p;
	case OPTIONAL:
		if (childNode) 
			return PATTERN.makeChoice(this.buildSeq(grammar, define, childNode, context) , EmptySequence);
		else
			return EmptySequence;
	case ZEROORMORE:
		if (childNode) 
			return PATTERN.makeChoice(PATTERN.makeOneOrMore(this.buildSeq(grammar, define, childNode, context)) , EmptySequence)
		else
			return EmptySequence;
	case CHOICE:
		return PATTERN.makeChoice(this.buildChoice(grammar, define, childNode, context) , EmptyChoice);
	case INTERLEAVE:
		return PATTERN.makeInterleave(this.buildInterleave(grammar, define, childNode, context) , EmptySequence);
	case MIXED:
		return PATTERN.makeInterleave(this.buildInterleave(grammar, define, childNode, context) , Text);
	case TEXT:
		return Text;
	case LIST:
		p = new ListPattern(theNode.getAttributeNS(XULAPP_EXTENSIONS,"separator"));
		p.p1 = this.buildSeq(grammar, define, childNode, context);
		return p;
	case EMPTY:
		return EmptySequence;
	case NOTALLOWED:
		return EmptyChoice;
	case NAME:
		this.getQName(theNode);
		var n = new NamePattern(QName.name, QName.ns);
		this.getQString(theNode, this.getText(theNode));
		n.name = QName.name;
		n.ns = QName.ns;
		return n;		
	case ANYNAME:
		p = new AnyNamePattern();		
		p.p1 = this.buildPattern(grammar, define, childNode, context);
		return p;
	case NSNAME:
		p = new NSNamePattern();	
		p.ns = this.getNS(theNode);
		p.p1 = this.buildPattern(grammar, define, childNode, context);
		return p;
	case EXCEPT:
		p = new ExceptPattern();
		p.p1 = this.buildSeq(grammar, define, childNode, context);
		return p;
	case DATA:
		p = new DataPattern(theNode.getAttribute("type"), this.getInheritedAttribute(theNode, "datatypeLibrary"));
		p.p1 = this.buildSeq(grammar, define, childNode, context);
		return p;
	case PARAM:
		p = new ParamPattern();
		p.value = this.getText(theNode);
		return p;
	case VALUE:
		p = new ValuePattern(theNode.getAttribute("type"), this.getInheritedAttribute(theNode, "datatypeLibrary"));
		p.value = this.getText(theNode);
		return p;
	case GRAMMAR:
		p = new GrammarPattern();
		p.parentGrammar = grammar;
		p.p1 = this.buildSeq(p, define, childNode, context);
		return p;
	case START:
		p = new StartPattern(grammar);
		p.p1 = this.buildSeq(grammar, define, childNode, context);
		return p;
	default:
		return new UnknownPattern();
	}
	return EmptyChoice;
}
RNGSchemaLoader.prototype.getText = function(theNode) {
	var childNode = theNode.firstChild;
	while (childNode) {
		if (childNode.nodeType == NODE_TYPE_TEXT || childNode.nodeType == NODE_TYPE_CDATA)
			return childNode.nodeValue;
		childNode = childNode.nextSibling;	
	}
	return "";
}
//Note QName is used as a global to reduce object creation.
RNGSchemaLoader.prototype.getQName = function(theNode) {
	QName.name = theNode.getAttribute("name");
	QName.ns = "";
	if (QName.name) {
		if (!this.getQString(theNode, QName.name)) {
			QName.ns = this.getNS(theNode, "ns");
		}
	}
}

RNGSchemaLoader.prototype.getNS = function(theNode) {
	var attrValue = "";
	do {
		attrValue = theNode.getAttribute("ns") || "";
		if (attrValue != "")
			break;
		if (theNode.tagName == ATTRIBUTE) // ns not inherited into attributes.
			break;
		theNode = theNode.parentNode;
	} 
	while (theNode && theNode.nodeType != NODE_TYPE_DOCUMENT);
	return attrValue;
}

RNGSchemaLoader.prototype.getInheritedAttribute = function(theNode, attrName) {
	var attrValue = "";
	do {
		attrValue = theNode.getAttribute(attrName) || "";
		if (attrValue != "")
			break;
		theNode = theNode.parentNode;
	} 
	while (theNode && theNode.nodeType != NODE_TYPE_DOCUMENT);
	return attrValue;
}

RNGSchemaLoader.prototype.getQString = function(theNode, qString) {
	var i = qString.indexOf(":");
	if (i != -1) { // namespace qualified name.
		QName.name = qString.substr(i+1);
		var prefix = qString.substr(0,i-1);
		QName.ns = theNode.lookupNamespaceURI(prefix);
		if (!QName.ns) 
			logger.log("Namespace prefix:" + prefix + "not declared");
		return true;	
	}
	else {
		QName.name = qString;
		QName.ns = "";
		return false;
	}
}

RNGSchemaLoader.prototype.nextElement = function(theNode) {
	while (theNode) {
		if (theNode.nodeType == NODE_TYPE_ELEMENT)
			return theNode;
		theNode = theNode.nextSibling;
	}			
	return null;
}
RNGSchemaLoader.prototype.buildSeq = function(grammar, define, theNode, context) {
	if (!theNode)
		return EmptySequence;
	var p;	
	var nextSibling;	
	if (context != INCLUDE && theNode.localName == DEFINE) {
		// using recursion for long lists of defines blows the javascript stack.
		nextSibling = theNode;
		do {
			p = this.buildPattern(grammar, define, nextSibling, context);
			nextSibling = this.nextElement(nextSibling.nextSibling);
		} while (nextSibling &&	nextSibling.localName == DEFINE);
	}
	else {
		p = this.buildPattern(grammar, define, theNode, context);
		nextSibling = this.nextElement(theNode.nextSibling);
	}	
	return (nextSibling)?
		PATTERN.makeSequence(p, this.buildSeq(grammar, define, nextSibling, context) ):
		p;
}
RNGSchemaLoader.prototype.buildChoice = function(grammar, define, theNode, context) {
	if (!theNode)
		return EmptyChoice;
	var p = this.buildPattern(grammar, define, theNode, context);
	var nextSibling = this.nextElement(theNode.nextSibling);
	return (nextSibling)?
		PATTERN.makeChoice(p, this.buildChoice(grammar, define, nextSibling, context) ):
		p;
}
RNGSchemaLoader.prototype.buildInterleave = function(grammar, define, theNode, context) {
	if (!theNode)
		return EmptySequence;
	var p = this.buildPattern(grammar, define, theNode, context);
	var nextSibling = this.nextElement(theNode.nextSibling);
	return (nextSibling)?
		PATTERN.makeInterleave(p, this.buildInterleave(grammar, define, nextSibling, context) ):
		p;
}	
RNGSchemaLoader.prototype.buildExcept = function(grammar, define, theNode, context) {
	if (!theNode)
		return EmptyChoice;
	var p = this.buildPattern(grammar, define, theNode, context);
	var nextSibling = this.nextElement(theNode.nextSibling);
	return (nextSibling)?
		PATTERN.makeExcept(p, this.buildExcept(grammar, define, nextSibling, context) ):
		p;
}

RNGSchemaLoader.prototype.mergeInclude = function(grammar, include) {
	var iGrammar = include.iGrammar;
	if (!grammar.start) 
		grammar.start = iGrammar.start;
	include.iGrammar = null;
	// replace include with div.
	var d = new DivPattern(include.p1);
	
	// if the include contains a start pattern then
	// remove any start patterns from the included grammar.
	// TODO check that include also contains the same start.
	var starts = d.getElementsByTagName(START);
	if (starts.length > 0) {
		var iStarts = iGrammar.getElementsByTagName(START);
		for (var i=0; i < iStarts.length; i++) {
			if (iStarts[i].p1.patName == START)
				iStart[i].p1 = EmptySequence;
			if (iStarts[i].p2.patName == START)
				iStart[i].p2 = EmptySequence;
		}
	}

	// if the include contains any define patterns then
	var defines = d.getElementsByTagName(DEFINE);
	if (defines.length > 0) {
		var iDefines = iGrammar.getElementsByTagName(DEFINE);
		// remove any like named patterns from the included grammar.
		for (i=0; i < defines.length; i++) {
			var define = defines[i].p1;
			var iDefine = iGrammar.defineMap.get(define.name);
			assert(iDefine,"Reference to missing define from include:" + nl + "href:" + include.href + nl + "name:" + define.name);
			iGrammar.defineMap.remove(define.name);
			// now back insert the define into the included grammar.
			iGrammar.defineMap.put(define.name, define);
		}
	}
	
	// Now merge the included defines.
	var keys = iGrammar.defineMap.keys();
	for (i=0; i < keys.length; i++) {
		iDefine = iGrammar.defineMap.get(keys[i]);
		if (grammar.defineMap.exists(iDefine.name)) {
			this.combineDefine(grammar, iDefine)
		}
		else {
			grammar.defineMap.put(iDefine.name, iDefine);
		}
	}
	d.p1 = iGrammar.p1;
	d.baseURI = iGrammar.baseURI;
	// If all the defines have been combined the div may now be empty - so remove it.
	if (!d.p1)
		d = null;
	return d;
}

RNGSchemaLoader.prototype.combineDefine = function(grammar, newDefine) {
	var baseDefine = grammar.defineMap.get(newDefine.name);
	if (!baseDefine.combine && newDefine.combine)
		baseDefine.combine = newDefine.combine;
	if (!newDefine.combine && baseDefine.combine)
		newDefine.combine = baseDefine.combine;
	assert(baseDefine.combine == newDefine.combine && baseDefine.combine != null,
		"Incompatible combine options: " + newDefine.name + " (old:" + baseDefine.combine + ", new:" + newDefine.combine + ")");
	assert(newDefine.p1,"new define has no content");
	switch (baseDefine.combine) {
	case CHOICE:
		baseDefine.p1 = convertToChoice(baseDefine.p1, newDefine.p1);
		break
	case INTERLEAVE:
		baseDefine.p1 = convertToInterleave(baseDefine.p1, newDefine.p1);
		break;
	default:
		assert(false, "Invalid combine option.");
	}
	newDefine.p1 = null;
}

RNGSchemaLoader.prototype.extractBaseURI = function(uri) {
	var i = uri.lastIndexOf("/");
	if (i != -1) {
		return uri.substr(0,i+1);
	}
	return uri;
}

RNGSchemaLoader.prototype.normalize = function(grammar) {
	// remove unreachable defines.
	assert(grammar.start,"Grammar does not have a start element.");
	this.tagReachableDefines(grammar, grammar.start);
	var removedDefines = "";
	var keys = grammar.defineMap.keys();
	for (var i=0; i < keys.length; i++) {
		var define = grammar.defineMap.get(keys[i]);
		if (!define.referenced) {
			grammar.defineMap.remove(define.name);
			removedDefines += nl + define.name;
		}
	}
	if (removedDefines != "")
			debug("Removed unreachable defines:" + removedDefines);
	
	// propogate nullable references back up the tree.
	grammar.visitID++;
	this.patchIsNullable(grammar, grammar.start);
}

RNGSchemaLoader.prototype.tagReachableDefines = function(grammar, pattern) {
	if (!pattern)
		return;
	if (pattern.patName == REF) {
		var define = grammar.defineMap.get(pattern.name);
		assert(define, "Reference to undefined define:" + pattern.name);
		pattern.refDefine = define;
		if (!define.referenced) {
			define.referenced = true;
			this.tagReachableDefines(grammar, define.p1);
		}	
		return;
	}
	else
		if (pattern instanceof UnaryPattern) {
			this.tagReachableDefines(grammar, pattern.p1);
		}
		else
			if (pattern instanceof BinaryPattern) {	
				var patName =  pattern.patName;
				var p = pattern;
				do {
					this.tagReachableDefines(grammar, p.p1);
					p = p.p2;
				} while (p && p.patName == patName);	
				this.tagReachableDefines(grammar, p);
			}	
}

// grammar.visisID++ required for first call.
RNGSchemaLoader.prototype.patchIsNullable = function(grammar, pattern) {
	//debug("patch:" + pattern.patName + " " + (("name" in pattern)?pattern.name:""));
	if (!pattern)
		return;

	if (pattern.patName == REF) {
		var define = pattern.refDefine;
		if (define.visitID != grammar.visitID) {
			define.visitID = grammar.visitID;
			this.patchIsNullable(grammar, define);
		}
		pattern.isNullable = define.isNullable;
		return;
	}
	else
		if (pattern instanceof UnaryPattern) {
			if (!(pattern instanceof AttributePattern))
				this.patchIsNullable(grammar, pattern.p1);
			if (!(pattern instanceof ElementPattern || pattern instanceof AttributePattern)) 
				pattern.isNullable = pattern.p1.isNullable;
		}
		else
			if (pattern instanceof BinaryPattern) {	
				this.patchIsNullable(grammar, pattern.p1);			
				this.patchIsNullable(grammar, pattern.p2);
				if (pattern instanceof ChoicePattern)
					pattern.isNullable = pattern.p1.isNullable || pattern.p2.isNullable;
				else	
					pattern.isNullable = pattern.p1.isNullable && pattern.p2.isNullable;
			}	
}
