/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
 
function SchemaSerializer() {
	nl = Utility.NEWLINE;
	this.dumpInternal = true;
}
SchemaSerializer.prototype.serialize = function(grammar) {
	var out = {xml:""};
	this.serializePattern(out, grammar, 0, false);
	return out;
}

SchemaSerializer.prototype.serializePattern = function(out, pattern, level) {
	if (pattern == null)
		return;
	var extra;
	if (!this.dumpInternal && pattern.patName == CHOICE) {
		if (pattern.p2 == EmptySequence)
			pattern = new OptionalPattern(pattern.p1);
	}
	out.xml += this.indent(level);
	out.xml += "<" + pattern.patName;  
	var attrs = pattern.attributesToString();
	if (attrs)
		out.xml += " " + attrs;
	extra = ""	
	if (this.dumpInternal && pattern.isNullable) extra += "@"

	if (pattern instanceof UnaryPattern && (!(pattern.p1 == EmptyChoice || pattern.p1 == EmptySequence)) ) {
		if (pattern.p1 || pattern.name instanceof Pattern) {
			out.xml += ">" + extra + nl;
			// NAME NSNAME or ANYNAME
			if (pattern.name instanceof Pattern) {
				this.serializePattern(out, pattern.name , level + 1);
			}
			this.serializePattern(out, pattern.p1 , level + 1);

			if (pattern instanceof GrammarPattern) {
				// Defines are not linked to grammar element so output from defineMap:
				out.xml += nl;
				var keys = pattern.defineMap.keys();
				for (var i=0; i<keys.length; i++) {
					out.xml += nl;
					this.serializePattern(out, pattern.defineMap.get(keys[i]) , level + 1);
				}
			}
			out.xml += this.indent(level);
			out.xml += "</" + pattern.patName +  ">" + nl;
		}
		else
			out.xml += "/>" + extra + nl;
	}
	else 
		if (pattern instanceof BinaryPattern) {
			out.xml += ">" + extra + nl;
			var p = pattern;
			var patName =  pattern.patName;
			do {
				this.serializePattern(out, p.p1, level + 1)
				p = p.p2;
			} while (p && p.patName == patName);
			this.serializePattern(out, p, level + 1)
			out.xml += this.indent(level);
			out.xml += "</" + pattern.patName +  ">" + nl;
			
		}
		else {
			switch (pattern.patName) {
			case NAME:
				out.xml += ">" + extra;
				out.xml += pattern.name;
				out.xml += "</" + pattern.patName +  ">" + nl;
				break;			
			case VALUE:
			case PARAM:
				out.xml += ">" + extra;
				out.xml += pattern.value;
				out.xml += "</" + pattern.patName +  ">" + nl;
				break;			
			default:
				out.xml += "/>" + extra + nl;
			}
		}	
}

SchemaSerializer.prototype.indent = function(level) {
	return tabs.substr(0,level);
}




