/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
 
netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");

function TokenDef(name) {
	this.name = name;
}
function Token() {
}
function TokenizerState(name) {
	this.name = name;
	this.tokenDefs = {};
	this.tokenDefs.__proto__ = null; // use as hash table.
	this.variable = [];
	this.variable.push(""); // dummy entry to lineup with regex results array.
}

function Tokenizer(grammarDoc) {
	this.states = {};
	this.states.__proto__ = null; // use as hash map.
	
	// load hash table of regex macro definitions.
	var macroDefs = {};
	macroDefs.__proto__ = null; // use as hash table.
	var defs = grammarDoc.getElementsByTagName("def");
	for (var i=0; i<defs.length; i++) {
		var def = defs.item(i);
		macroDefs[def.getAttribute("name")] = def.getAttribute("as");
	}

	// Build hash table of states.
	var statesList = grammarDoc.getElementsByTagName("state");
	for (var i=0; i<statesList.length; i++) {
		var elemState = statesList.item(i);
		var stateName = elemState.getAttribute("name");
		var state = new TokenizerState(stateName);
		this.states[stateName] = state;
	}
	
	// Build a separate regex for token in each state.
	for (var i=0; i<statesList.length; i++) {
		var state = this.states[stateName];
		var elemState = statesList.item(i);
		this.buildStateRegEx(macroDefs, elemState);
	}
	

	
	this.EOF = new TokenDef("EOF");
	

}

Tokenizer.prototype.buildStateRegEx = function(macroDefs, elemState) {
	var stateName = elemState.getAttribute("name");
	var state = this.states[stateName];

	var isMacroDef = /{([^}]+)}/g; // match macro def use.

	var re = "";
	var orSep = "";

	var tokens = elemState.getElementsByTagName("token");
	for (i=0; i<tokens.length; i++) {
		var t = tokens.item(i);
		var tokenDef = new TokenDef(t.getAttribute("name"));

		var thenMatch = t.getAttribute("thenmatch");
		if (thenMatch)
			tokenDef.thenMatch = new RegExp(thenMatch,"gm");

		// Request Transfer to new state after matching this token.
		var nextStateName = t.getAttribute("nextstate");
		if (nextStateName) {
			var nextState = this.states[nextStateName];
			if (nextState)
				tokenDef.nextState = nextState;
			else
				throw "nextstate:" + nextStateName + " refers to undefined state.";
		}

		// Ignore input matching this token
		var ignore = t.getAttribute("ignore");
		if (ignore) {
			if (ignore == "true")
				tokenDef.ignore = true;
		}
		
		// Lookup value of this token in hash table of tokens to find actual token.
		if (t.hasAttribute("id")) {
			var id = t.getAttribute("id");
			state.tokenDefs[id] = tokenDef;
		}
		
		// Variable length tokens will be matched by wrapping the regex in parentheses.
		// The first regex.results[] entry is then mapped to the corresponding tokendef.
		var isVariable = t.hasAttribute("variable");
		if (isVariable) {
			state.variable.push(tokenDef);
		}
		
		if (t.hasAttribute("match")) {
			var match = t.getAttribute("match");
			// replace any macro defs.
			var result = "X";
			var newMatch = match;
			isMacroDef.lastIndex = 0;
			while (result) {
				result = isMacroDef.exec(match);
				if (result) {
					for (var j=1; j<result.length; j++) {
						var defName = result[j];
						if (macroDefs[defName] != undefined) {
							newMatch = newMatch.replace(new RegExp("{" + defName + "}","g"), macroDefs[defName]);
						}
					}	
				}
			}
			match = newMatch;
			if (isVariable)
				re += orSep + "(" + match + ")"; // group variable length patterns so we can find them.
			else	
				re += orSep +  match;
			orSep = "|"
		}
	}

	state.regex = new RegExp(re,"gm");
}
Tokenizer.prototype.init = function(text) {
	this.text = text;
	var stateName = "NORMAL";
	this.state = this.states[stateName];
	this.regex = this.state.regex;
	this.regex.lastIndex = 0;
}

Tokenizer.prototype.nextToken = function(token) {
	var regex = this.regex;
	var state = this.state;
	var text = this.text;
	while (regex.lastIndex < text.length) {
		var	results = regex.exec(text); 
		if (results) {
			token.val = results[0];
			token.offset = regex.lastIndex - token.val.length;
			var t = null;
			t = state.tokenDefs[token.val];
			if (!t) {
				// lookup variable length patterns for match.
				// first match is taken ==> Put more complex patterns first in definitions.
				for (var i=1; i<results.length; i++) {
					if (results[i]) {
						t = state.variable[i];
						break;
					}
				}
			}

			if (t) {
				if (t.ignore)
					continue;
				token.type = t;	
				if (t.thenMatch) {
					t.thenMatch.lastIndex = regex.lastIndex;
					results = t.thenMatch.exec(this.text);
					token.val += results[0];
					token.offset = t.thenMatch.lastIndex - token.val.length;
					regex.lastIndex = t.thenMatch.lastIndex;
				}
				if (t.nextState) {
					this.currentState = t.nextState;
					var i = regex.lastIndex;
					this.state = this.states[this.currentState];
					this.regex = regex = state.regex;
					regex.lastIndex = i; // resume where previous regex finished.
				}
			}
			else {
				// error
				var i = regex.lastIndex;
				this.state = this.states["NORMAL"];
				this.regex = regex = state.regex;
				regex.lastIndex = i; // resume where previous regex finished.
			}
		}
		else {
			// error
		}
		return;
	}
	token.type = this.EOF;
	token.val = null;
	token.offset = text.length;
}

