/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
 
function PatternViewDeriv(theViewNode, level, errorList) {
	//logger.log ("<" + theViewNode.nodeName + "> " + this.patName + " (Pattern)", level);
	return EmptyChoice;
}

function StartViewDeriv(theViewNode, level, errorList) {
	//logger.log ("<" + theViewNode.nodeName + "> " + this.patName, level);
	return this.p1.viewDeriv(theViewNode, level, errorList);
}
function RefViewDeriv(theViewNode, level, errorList) {
	//logger.log ("<" + theViewNode.nodeName + "> " + this.patName + " name=" + this.name, level);
	return this.refDefine.p1.viewDeriv(theViewNode, level, errorList);
}
function DefineViewDeriv(theViewNode, level, errorList) {
	//logger.log ("<" + theViewNode.nodeName + "> " + this.patName + " name=" + this.name, level);
	return this.p1.viewDeriv(theViewNode, level, errorList);
}

function ElementViewDeriv(theViewNode, level, errorList) {
	//logger.log ("<" + theViewNode.nodeName + "> " + this.patName + " name=" + this.name, level);
	if (!theViewNode)
		return EmptyChoice;
	
	if (!this.matchViewNamespace(theViewNode))
		return EmptyChoice;

	//errorList.log(theViewNode, "@@ELEM:" + theViewNode.localName + " " +  theViewNode.getAttribute("name") + " Begin ", level);
		
	// Validate attributes:	
	var p = this.p1;
	var d = p;
	var ats = getAtsFromEl(theViewNode);
	var attributes = ats.getElementsByTagName("at");
	for (var i=0; i<attributes.length; i++) {
		var at = attributes.item(i);
		var an = getAnFromAt(at);
		if (an.firstChild.nodeValue.substr(0,5) != "xmlns") {
			d = d.viewDeriv(at, level + 1, errorList);
			if (d==EmptyChoice)
				errorList.log(at, "  @@ATTR:" + an.firstChild.nodeValue + " Invalid");
			//var out = serializer.serialize(d);
			//logger.log(out.xml, level + 1);
		}	
	}

	// Validate children
	var children = getExpFromEl(theViewNode);
	if (children) {
		var childNode = children.firstChild
		//var d = p;
		while (childNode) {
			if (childNode.nodeName == "hr" || childNode.nodeName == "cm") {
					childNode = childNode.nextSibling;
					continue; 
			}
			if (childNode.nodeName == "tx" || childNode.nodeName == "cd") {
				var isWS = RE_Whitespace.test(childNode.firstChild.nodeValue);
				if (!isWS) {
					d = d.viewDeriv(childNode, level + 1, errorList);
					if (d == EmptyChoice) {
						errorList.log(childNode, "  @@TEXT:" + childNode.firstChild.nodeValue + " Invalid");
					}	

				}	
				childNode = childNode.nextSibling;
				continue; 
			}
			d = d.viewDeriv(childNode, level + 1, errorList);
			if (d == EmptyChoice) {
				errorList.log(childNode, "  @@CHILD ELEM:" + getElLocalName(childNode) + " Invalid");
			}
			childNode = childNode.nextSibling;
		}
	}	
	//errorList.log(theViewNode, "@@ELEM:" + theViewNode.localName + " " +  theViewNode.getAttribute("name") +  " End: nullable=" + d.isNullable, level);
	return (d.isNullable)? EmptySequence : EmptyChoice;
}

function ElementMatchViewNamespace(theViewNode) {
	if (theViewNode.nodeName != "el")
		return false;
	// a namespace name.	
	if (this.name instanceof Pattern)
		return this.name.matchViewNamespace(theViewNode);
	// a simple name.
	//alert("mns:" + theViewNode.tagName + "  " + this.name);
	if (getElLocalName(theViewNode) != this.name)
		return false;
	var namespaceURI = getElNamespaceURI(theViewNode);	
	if (!namespaceURI)
		return (this.ns == "");
	return (namespaceURI == this.ns);
}

function AttributeViewDeriv(theViewNode, level, errorList) {
	//logger.log ("<" + theViewNode.nodeName + "> " + this.patName + " name=" + this.name, level);
	if (!theViewNode)
		return EmptyChoice;
	if (!this.matchViewNamespace(theViewNode))
		return EmptyChoice;
	if (this.p1)	
		return this.p1.viewDeriv(theViewNode,level, errorList)		
	return EmptySequence;
}
function AttributeMatchViewNamespace(theViewNode) {
	if (theViewNode.nodeName != "at")
		return false;
	// a namespace name.	
	if (this.name instanceof Pattern)
		return this.name.matchViewNamespace(theViewNode);
	// a simple name.
	var an = getAnFromAt(theViewNode);
	var attrName = an.firstChild.nodeValue;
	var i = attrName.indexOf(":");
	if (i == -1) 
		return (attrName == this.name);
	attrName = attrName.substr(i + 1);
	if (attrName != this.name)
		return false;
	var prefix = attrName.substr(0, i);
	var el = theViewNode.parentNode.parentNode; // at->ats->el.
	var namespaceURI = getElNamespaceURI(el, prefix);
	return (namespaceURI == this.ns);
}

function OneOrMoreViewDeriv(theViewNode, level, errorList) {
	//logger.log ("<" + theViewNode.nodeName + "> " + this.patName, level);
	var d1 = this.p1.viewDeriv(theViewNode,level, errorList);
	return d1.makeSequence(	d1, 
								d1.makeChoice(d1.makeOneOrMore(this.p1, level), EmptySequence, level), level );
}
function ChoiceViewDeriv(theViewNode, level, errorList) {
	//logger.log ("<" + theViewNode.nodeName + "> " + this.patName, level);
	var d1 = this.p1.viewDeriv(theViewNode, level, errorList);
	var d2 = this.p2.viewDeriv(theViewNode, level, errorList);
	return d1.makeChoice(d1, d2, level);
}
function ChoiceMatchViewNamespace(theViewNode) {
	return (this.p1.matchViewNamespace(theViewNode) || this.p2.matchViewNamespace(theViewNode));
}

function SequenceViewDeriv(theViewNode, level, errorList) {
	//logger.log ("<" + theViewNode.nodeName + "> " + this.patName, level);
	if (!theViewNode)
		return EmptyChoice;
	var d1, d2;
	if (theViewNode.nodeName == "at") {
		d1 = this.p1.viewDeriv(theViewNode, level, errorList);
		d2 = this.p2.viewDeriv(theViewNode, level, errorList);
		return d1.makeChoice(
			d1.makeInterleave( d1,	this.p2, level),
			d1.makeInterleave( this.p1, d2, level), level);
	}
	d1 = this.p1.viewDeriv(theViewNode, level, errorList);
	var tem = d1.makeSequence(	d1, this.p2,level);
	if (this.p1.isNullable) {
		d2 = this.p2.viewDeriv(theViewNode, level, errorList);
		return d1.makeChoice(tem, d2, level);
	}	
	else	
		return tem;
}
// This function is called only within Except Patterns.
function SequenceMatchViewNamespace(theViewNode) {
	return ( this.p1.matchViewNamespace(theViewNode) || this.p2.matchViewNamespace(theViewNode) );
}
function InterleaveViewDeriv(theViewNode, level, errorList) {
	//logger.log ("<" + theViewNode.nodeName + "> " + this.patName, level);
	var d1 = this.p1.viewDeriv(theViewNode, level, errorList);
	var d2 = this.p2.viewDeriv(theViewNode, level, errorList);
	return d1.makeChoice(
		d1.makeInterleave(d1, this.p2, level),
		d1.makeInterleave(this.p1, d2, level), level);
}

function ValueViewDeriv(theViewNode, level, errorList) {
	if (!theViewNode)
		return EmptyChoice;

	if (theViewNode.nodeName == "at") {
		if (datatypeLibraries.datatypeEquals(	this.datatypeLibrary,
															(this.type || "string"),
															getAvFromAt(theViewNode).firstChild.nodeValue,
															null,
															this.value,
															null) )
			return EmptySequence;
		else
			return EmptyChoice;
	}
	if (theViewNode.nodeName == "tx" || theViewNode.nodeName == "cd") {
		if (datatypeLibraries.datatypeEquals(	this.datatypeLibrary,
															(this.type || "string"),
															theViewNode.firstChild.nodeValue,
															null,
															this.value,
															null) )
			return EmptySequence;
		else
			return EmptyChoice;
	}
	return EmptyChoice;
}

function DataViewDeriv(theViewNode, level, errorList) {
	if (!theViewNode)
		return EmptyChoice;

	if (theViewNode.nodeName == "at") {
		if (datatypeLibraries.datatypeAllows(	this.datatypeLibrary,
															(this.type || "string"),
															getAvFromAt(theViewNode).firstChild.nodeValue,
															null) ) //TODO pass param's
			return EmptySequence;
		else
			return EmptyChoice;
	}
	if (theViewNode.nodeName == "tx" || theViewNode.nodeName == "cd") {
		if (datatypeLibraries.datatypeAllows(	this.datatypeLibrary,
															(this.type || "string"),
															theViewNode.firstChild.nodeValue,
															null) ) //TODO pass param's
			return EmptySequence;
		else
			return EmptyChoice;
	}
	return EmptyChoice;
}
 
function TextViewDeriv(theViewNode, level, errorList) {
	if (theViewNode.nodeName == "tx" || theViewNode.nodeName == "cd") 
		return Text;
	else
		return EmptyChoice;
}

var tempAttr = null;
function ListViewDeriv(theViewNode, level, errorList) {
	if (!tempAttr) { 
		tempAttr = document.createElementNS(XUL, "tx");
		tempAttr.appendChild(document.createTextNode(""));
	}	
	if (theViewNode.nodeName == "at" || theViewNode.nodeName == "tx" || theViewNode.nodeName == "cd") {
		if (theViewNode.nodeName == "at")
			var tempAttrValue = getAvFromAt(theViewNode).firstChild.nodeValue;
		else	
			tempAttrValue = theViewNode.firstChild.nodeValue;
		var vals = tempAttrValue.split(this.separatorRegExp);
		for (var i=0; i<vals.length; i++) {
			tempAttr.firstChild.nodeValue = vals[i];
			var d = this.p1;
			d = d.viewDeriv(tempAttr, level, errorList);
			if (d == EmptyChoice)
				return EmptyChoice;
		}
		return EmptySequence;
	}
	return EmptyChoice;
}

function NameMatchViewNamespace(theViewNode) {
	if (theViewNode.localName != this.name || (theViewNode.namespaceURI||"") != this.ns)
		return false;
	if (this.p1 == EmptySequence)
		return true;
	return this.p1.matchViewNamespace(theViewNode);	
}

function AnyNameMatchViewNamespace(theViewNode) {
	if (this.p1 == EmptyChoice)
		return true;
	return this.p1.matchViewNamespace(theViewNode);	
}

function NSNameMatchViewNamespace(theViewNode) {
	if ((theViewNode.namespaceURI||"") != this.ns)
		return false;
	if (this.p1 == EmptyChoice)
		return true;
	return this.p1.matchViewNamespace(theViewNode);	
}

function ExceptMatchViewNamespace(theViewNode) {
	return (!this.p1.matchViewNamespace(theViewNode));	
}

function EmptySequenceViewDeriv(theViewNode, level, errorList) {
	//logger.log ("<" + theViewNode.nodeName + "> " + this.patName, level);
	return EmptyChoice;
}

