/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
 
netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");

var xulapp = window.arguments[0];
var xulDoc = window.arguments[1];
var styleSheetNode = window.arguments[2]; // the selected styleSheet or null. Null indicates use last styleSheet specified.
var editorWindow = xulDoc.editorWindow;
var theSrcNode;
var regexpStruct = new Object();

var cssRuleType = ["UNKNOWN_RULE", "STYLE_RULE", "CHARSET_RULE", "IMPORT_RULE", "MEDIA_RULE", "FONT_FACE_RULE", "PAGE_RULE"];


function OnLoad(event) {
	clearRules();
	loadRules();
}

function clearRules() {
	// clear old rules.
	var treeChildren = document.getElementById("rulesList");
	while (treeChildren.hasChildNodes()) {
		treeChildren.removeChild(treeChildren.firstChild); 
	}
}

function loadRules() {
	var treeChildren = document.getElementById("rulesList");
	
	var styleList = editorWindow.styleSheets;
	if (styleList.length == 0)
		return;
	for (var i=0; i<styleList.length; i++) {
		var styleSheet = styleList.item(i);
		if (styleSheet.type != "text/css") 
			continue; // DSSL perhaps?
		cssStyleSheet = styleSheet.QueryInterface(Components.interfaces.nsIDOMCSSStyleSheet);
		// ignore style sheets defined by xulEditor.
		if (editorWindow.isKnownStyleSheet(cssStyleSheet.href))
			continue;
		document.documentElement.setAttribute("title" , cssStyleSheet.href);
		var cssRules = cssStyleSheet.cssRules;
		for (var j=0; j<cssRules.length; j++) {
			var cssRule = cssRules.item(j);
			if (cssRule.type != Components.interfaces.nsIDOMCSSRule.STYLE_RULE)
				continue;
			treeChildren.appendChild(buildRuleItem(cssRule));			
		}

	}
	
	
	
}

function buildRuleItem(cssRule) {
	var treeitem = document.createElement("treeitem");
	treeRow = document.createElement("treerow");
		treeCell = document.createElement("treecell");
			treeCell.setAttribute("label", cssRule.selectorText);
			treeCell.setAttribute("flex","1");
			treeRow.appendChild(treeCell);
		treeCell = document.createElement("treecell");
			//treeCell.setAttribute("allowevents","true"); // otherwise included widgets will be UI dead.
			treeCell.setAttribute("flex","1");
			treeCell.setAttribute("label", cssRule.style.cssText);
				//var textbox = document.createElement("textbox");
				//textbox.setAttribute("value", cssRule.cssText);
				//textbox.setAttribute("onblur", "textPropertyBlur(event);");
				//textbox.setAttribute("flex","1");
			//treeCell.appendChild(textbox);
			treeRow.appendChild(treeCell);
	treeitem.appendChild(treeRow);

	return treeitem;
}

function checkboxPropertyChanged(event) {
	var checkbox = event.target;
	if (checkbox.localName != "checkbox")
		return;
	var value = (checkbox.getAttribute("checked")=="true") ? "true" : "false";
	updateAttribute(checkbox.attributeid, value);
}

function menulistPropertyChanged(event) {
	var menuitem = event.target;
	if (menuitem.localName != "menuitem")
		return;
	var menupopup = menuitem.parentNode; // menuitem->menupopup.
	if (menupopup.localName != "menupopup")
		return;

	updateAttribute(menupopup.attributeid, menuitem.getAttribute("label"));
}

function commentBlur(event) {
	var inputElement = event.target; // an html:input object.
	var oldValue = theSrcNode.data;
	if (oldValue != inputElement.value) {
		theSrcNode.data = inputElement.value;
		xulapp.currentDocument.notifyAttributeChanged(theSrcNode, null, oldValue, theSrcNode.data, window);
	}	
}


function PIPropertyBlur(event) {
	var inputElement = event.target; // an html:input object.
	var oldValue = theSrcNode.data;
	if (oldValue != inputElement.value) {
		theSrcNode.data = inputElement.value;
		xulapp.currentDocument.notifyAttributeChanged(theSrcNode, null, oldValue, theSrcNode.data, window);
	}	
}

function textPropertyBlur(event) {
	var inputElement = event.target; // an html:input object.
	// find owning textbox:
	var textbox = inputElement; // start here just in case this changes in the future.
	while (textbox.localName != "textbox") {
		textbox = textbox.parentNode;
		if (!textbox) break; // no textbox parent!
	}
	if (!textbox) return;

	// XXX apply RegExp to check value.
	getAttributeRegexp(textbox.attributeid, regexpStruct);
	if (regexpStruct.regexp) {
		//debug("regexp:" + regexpStruct.regexp);
		var a = inputElement.value.match(regexpStruct.regexp);
		if (!a) {
			debug("Attribute:" + textbox.attributeid + " value:" + inputElement.value + "\n" + regexpStruct.errmsg);
			textbox.focus();
			return;
		}
	}
	// XXX we should prevent empty string.

	updateAttribute(textbox.attributeid, inputElement.value);
}

function updateAttribute(attributeid, newValue) {
	var oldValue = theSrcNode.getAttribute(attributeid);

	// only change attribute if value has changed = minimize DOM mutation events.
	if (oldValue != newValue) {
		//debug(attributeid + " Old:" + oldValue + " New:" + newValue);
		theSrcNode.setAttribute(attributeid, newValue);
		xulapp.currentDocument.notifyAttributeChanged(theSrcNode, attributeid, oldValue, newValue, window);		
	}

}

function deleteRule(event) {
	tree = event.target;
	if (tree.localName != "tree") return;
	if (tree.selectedItems.length <= 0) return;
	treeitem = tree.selectedItems[0];

	var attributeid = treeitem.attributeid;
	var oldValue = theSrcNode.getAttribute(attributeid);
	theSrcNode.removeAttribute(attributeid);
	xulapp.currentDocument.notifyAttributeChanged(theSrcNode, attributeid, oldValue, null, window);
	var treechildren = treeitem.parentNode;
	treechildren.removeChild(treeitem);
}

function getAttributeRegexp(attributeid, regexpStruct) {
	var regexp = null
	regexpStruct.regexp = null;
	regexpStruct.errmsg = null;
	var attributeDef = getAttributeDef(theSrcNode.localName, attributeid);
	regexp = attributeDef.getAttribute("regexp")
	regexpStruct.errmsg = attributeDef.getAttribute("errmsg")
	if (!regexpStruct.regexp) {
		// Does the attribute type have a regexp.
		var attributeType = attributeDef.getAttribute("type");
		if (attributeType) {
			var attributeTypeDef = xulapp.widgetAttributes.getElementById("type-" + attributeType);
			regexp = attributeTypeDef.getAttribute("regexp");
			regexpStruct.errmsg = attributeTypeDef.getAttribute("errmsg");
		}
	}
	// parse and convert regexp to compiled re
	if (regexp) {
		var i = regexp.lastIndexOf("/");
		var re = regexp.substring(1,i); // drop "/" off pattern
		if (i == (regexp.length -1)) {
			regexpStruct.regexp = new RegExp(re);
		}
		else {
			var flags = regexp.substr(i+1);		
			regexpStruct.regexp = new RegExp(re, flags);
		}

	}
}

function buildAttributesPopup(event, loadEventHandlers) {
	if (!theSrcNode)
		return false; // no popup.
	var menupopup = event.target;
	// clear popup.
	while (menupopup.hasChildNodes()) {
		menupopup.removeChild(menupopup.firstChild);
	}
	
	var widgetDef = xulapp.widgets.getElementById(theSrcNode.localName);
	if (!widgetDef)
		return false;

	// get widgets attributes element
	var attributes = widgetDef.getElementsByTagName("attributes");
	if (attributes.length <= 0)
		return false;

	// load inherited attributes.
	var inherit = attributes[0].getAttribute("inherit");
	if (inherit) {
		var inheritWidgetDef = xulapp.widgets.getElementById(inherit);
		if (inheritWidgetDef) {
			inheritAttributes =	inheritWidgetDef.getElementsByTagName("attribute");
			buildAttributeMenuItem(menupopup, inheritAttributes, loadEventHandlers);
			var menuseparator = document.createElement("menuseparator");
			menupopup.appendChild(menuseparator);
		}
	}
	var widgetAttributes = widgetDef.getElementsByTagName("attribute");
	buildAttributeMenuItem(menupopup, widgetAttributes, loadEventHandlers );
	return true;
}

function buildAttributeMenuItem(menupopup, attributesNodeList, loadEventHandlers) {
	for (var i=0; i<attributesNodeList.length; i++) {	
		var menuitem = document.createElement("menuitem");
		var attributeRef = attributesNodeList.item(i);
		var attributeid = attributeRef.getAttribute("attributeid");
		var attributeDef = getAttributeDef(theSrcNode.localName, attributeid);
		if (attributeDef.getAttribute("eventhandler")) {
			if (!loadEventHandlers) continue;
		}
		else {
			if (loadEventHandlers) continue;		
		}
		menuitem.attributeid = attributeid;	// cache	
		menuitem.setAttribute("label", attributeid);
		menupopup.appendChild(menuitem);
	}	
}
function buildAttributeValuesPopup(event) {
	if (!theSrcNode) 
		return false; // no popup.
	var menupopup = event.target;
	if (menupopup.localName != "menupopup")
		return;
	// clear popup.
	while (menupopup.hasChildNodes()) {
		menupopup.removeChild(menupopup.firstChild);
	}

	var attributeDef = getAttributeDef(theSrcNode.localName, menupopup.attributeid);
	var values = attributeDef.getAttribute("values");
	if (!values) 
		return false;

	var v = values.split(/\s/);
	for (var i=0; i<v.length; i++) {
		var menuitem = document.createElement("menuitem");
		menuitem.setAttribute("label", v[i]);
		menupopup.appendChild(menuitem);
	}	
	return true; // show popup.
}

function addSelectedAttribute(event) {
	if (!theSrcNode)
		return;
	var menuItem = event.target;
	if (menuItem.localName != "menuitem" )
		return;
	var attributeid = menuItem.attributeid;
	if (!theSrcNode.getAttribute(attributeid)) {
		var treeChildren = document.getElementById("rulesList");
		var attributeDef = getAttributeDef(theSrcNode.localName, attributeid);
		var values = attributeDef.getAttribute("values");
		value = ""
		if (values) {
			var v = values.split(/\s/);
			if (v.length > 0)
				value = v[0];
		}
		var treeitem = buildTreeItem(attributeid,value);
		treeChildren.appendChild(treeitem);	
		
		// For some attribute types we need to set the attribute value using the default value.
		// For these types the user may not change the value before leaving the widget and would expect
		// the displayed value to be reflected in the displayed form.
		attributeType = attributeDef.getAttribute("type");
		if (attributeType) attributeType != "string";
		switch (attributeType) {
		case "boolean":
			updateAttribute(attributeid, "false");
			break;
		case "list":
			updateAttribute(attributeid, value);
			break;
		default:
		}
	}
	selectAttribute(attributeid)

}

function findTreeItem(attributeid) {
	var treeChildren = document.getElementById("propertiesList");
	treeitems = treeChildren.getElementsByTagName("treeitem");
	for (var i=0; i<treeitems.length; i++) {
		if (treeitems.item(i).attributeid == attributeid)
			return treeitems.item(i);
	}
	return null;
}

// show attribute as selected in properties list.
function selectAttribute(attributeid) {
	var treeitem = findTreeItem(attributeid);
	if (!treeitem)
		return;
	var propertiesTree = document.getElementById("propertiesTree");
	// clear current selection
	for (var i=0; i < propertiesTree.selectedItems.length; i++) {
		propertiesTree.selectedItems[i].removeAttribute("selected")
	}
	
	treeitem.setAttribute("selected","true");
	propertiesTree.ensureElementIsVisible(treeitem);
}

function getAttributeDef(localName, attributeid) {
	var attributeDef = xulapp.widgetAttributes.getElementById(attributeid);
	if (attributeDef && attributeDef.hasChildNodes()) {
		var childNodes = attributeDef.getElementsByTagName("subattribute");
		for (var i=0; i<childNodes.length; i++) {
			attributeDef = childNodes[i];
			if (attributeDef.getAttribute("widget") == localName)
				break;
		}
	}	
	return attributeDef;
}

function OnUnload(event) {
	if (xulapp.widgetTreeEditor) {
		xulapp.widgetTreeEditor.btnShowStylesheetInspector.removeAttribute("checked");
		xulapp.widgetTreeEditor.stylesheetInspector = null;
	}
}
