/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
 
netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");

var dragService = Components.classes["@mozilla.org/widget/dragservice;1"].getService().QueryInterface(Components.interfaces.nsIDragService);

var xulapp = window.arguments[0];
var openTool = window.arguments[1]; 
var xulDoc = openTool.xulDoc;

var widgetTree = null;
var widgetTreeView = null;

const MIMETYPE_NEW_WIDGET = "text/x-xulapp-newwidget";
const MIMETYPE_OLD_WIDGET = "text/x-xulapp-oldwidget"; 

const DRAGACTION_NONE = 0;
const DRAGACTION_COPY = 1;
const DRAGACTION_MOVE = 2;
const DRAGACTION_LINK = 4;

const MAX_LABEL_LENGTH = 100;

// convenience object to bundle drop target and drop action.

var dropItem = null; // the item on which we are dropping.
var dropItemParentName = null;
var dropWhere = null;
var dragItem = null; // the item being dragged. (may be a new widget or an existing treeitem).
var dragItemName = null;

var ignoreWidgetTreeSelect = false;

var widgetbar = null;
var propInspector = null;
var stylesheetInspector = null;

var btnShowToolbox = null;
var btnShowPropertyInspector = null;
var btnShowStylesheetInspector = null;

function OnLoad(event) {
	btnShowToolbox = document.getElementById("button-showToolBox");
	btnShowPropertyInspector = document.getElementById("button-showPropertyInspector");
	btnShowStylesheetInspector = document.getElementById("button-showStylesheetInspector");
	widgetTree = document.getElementById("widgetTree");
	widgetTreeView = new DroppableTreeView(widgetTree.view);
	widgetTree.view = widgetTreeView;
	// Provide implementations of drag drop support.
	var view = widgetTree.view.wrappedJSObject;
	view.canDropBeforeAfter = canDropWidgetBeforeAfter;
	view.canDropOn = canDropWidgetOn;
	view.drop = dropWidget;
	
	if (xulapp.currentDocument) {
		OnFormSelected(xulapp.currentDocument);
	}
	loadBoxAcceptsChildren();
}
function loadBoxAcceptsChildren() {
	// this is used to determine if a widget can be dropped on another widget.
	var boxWidget = xulapp.widgets.getElementById("box");
	var acceptschildren = boxWidget.getAttribute("acceptschildren");
	xulapp.boxAcceptsChildren = acceptschildren.split(/\s/);
}

function saveFile(event) {
	if (xulDoc)
		xulDoc.save();
}

function saveFileAs(event) {
	if (xulDoc)
		xulDoc.saveAs();
}
function OnShowToolBox() {
    if (this.widgetbar)
		this.widgetbar.close();    
    else {    	
    	this.widgetbar = window.openDialog("chrome://xulapp/content/tools/xulEditor/widgetbar.xul", "widget bar", "resizable=yes,chrome,dialog=yes,titlebar,alwaysraised",xulapp);
    	btnShowToolbox.setAttribute("checked","true"); 
    }
}
function OnShowPropertyInspector() {
	if (this.propInspector)
		this.propInspector.close();
	else {	
		this.propInspector = window.openDialog("chrome://xulapp/content/tools/xulEditor/propInspector.xul", "propInspector", "resizable=yes,chrome,titlebar",xulapp);
		btnShowPropertyInspector.setAttribute("checked","true"); 
	}
}

function OnShowStylesheetInspector() {
	if (this.stylesheetInspector)
		this.stylesheetInspector.close();
	else {	
		// last param is srcNode if null then use last style sheet else selected.
		this.stylesheetInspector = window.openDialog("chrome://xulapp/content/tools/xulEditor/styleSheetInspector.xul", "styleSheetInspector", "resizable=yes,chrome,titlebar",xulapp, xulDoc, null); 
		btnShowStylesheetInspector.setAttribute("checked","true"); 
	}
}

// Tile all but project window
 function OnTileWindows(event) {
	var top = 0;
	var leftWidth = 0;
	this.moveTo(0,0);
	top += this.outerHeight;
	leftWidth = this.outerWidth;
	this.focus();
	if (this.propInspector) {
		this.propInspector.moveTo(0,top);
		if (!leftWidth)
			leftWidth = this.propInspector.outerWidth;
		this.propInspector.resizeTo(this.outerWidth, screen.availHeight - top);
		this.propInspector.focus();

	}
}

function OnTreeSelect(event) {
	if (ignoreWidgetTreeSelect)
		return;
 	reconcileSelection(getSelectedItems(widgetTree ));
}

// Build list of items currently selected.
function getSelectedItems(tree )
{
    var selectedItems = new Array();
    var selection = widgetTreeView.selection;                  
    if( selection )
    {
        var rangeCount = selection.getRangeCount();
    
        for( var i = 0; i < rangeCount; i++ )
        {
            var startIndex = new Object();
            var endIndex = new Object();
    
            selection.getRangeAt( i, startIndex, endIndex );
    
            for( var j = startIndex.value; j <= endIndex.value; j++ )
            {
            	//debug(j + "XXX" + startIndex.value + "XXX" + endIndex.value);
            	// tree.view.getItemAtIndex(int) does not take into account collapsed branches.
            	//var treeitems = tree.getElementsByTagName("treeitem");
				var treeitem = widgetTreeView.getItemAtIndex(j);
                selectedItems.push( treeitem );
            }
        }
    }

    return selectedItems;
}

function reconcileSelection(selectedItems) {
	// This mess is necessary because tree does not support an onselectionchanged event.
	// Assume selected items have a xulapp-id attribute.
	// Pass one notify items which were selected but now are not.
	for (i=0; i<xulDoc.selectedNodeIds.length; i++) {
		var found = false;
		// is the node still in the selection?
		for (var j=0; j<selectedItems.length; j++) {
			var id = selectedItems[j].getAttribute("xulapp-id");
			if (xulDoc.selectedNodeIds[i] == id)  {
				found = true;
				break;
			}
		}
		if (!found) {
			// OK it's gone.
			xulDoc.notifyWidgetSelected(xulDoc.selectedNodeIds[i], false, false);
		}
	}
	// Pass two: and now the other way round.
	for (var i=0; i<selectedItems.length; i++) {
		var item = selectedItems[i];
		var id = item.getAttribute("xulapp-id");
		var found = false;
		// is the node still in the selection?
		for (var j=0; j<xulDoc.selectedNodeIds.length; j++) {
			if (xulDoc.selectedNodeIds[i] == id)  {
				found = true;
				break;
			}	
		}
		if (!found) {
			// OK it's new.
			xulDoc.notifyWidgetSelected(id, true, true);
			augmentSelection = false;
		}
	}
}

function OnFormSelected(newxulDoc) {
	if (xulDoc) {
		xulDoc.unregisterSelectionListener(OnWidgetSelected);
		xulDoc.unregisterAttributeListener(OnAttributeChanged);	
	}	
	xulDoc = newxulDoc;
	var win = document.getElementById("widgetTreeEditor");
	var title;
	loadTreeFromDOM(xulDoc);
	if (xulDoc) {
		xulDoc.registerSelectionListener(OnWidgetSelected);	
		xulDoc.registerAttributeListener(OnAttributeChanged);	
		// I can't get title modifier to work.
		title = "XUL App - " + xulDoc.path.substr(xulapp.project.projectDirectoryURI.length);
	}
	else
		title = "XUL App - No Document."
	win.setAttribute("title", title);
}

function OnWidgetSelected(nodeId, isSelected, augmentSelection) {
	if (!isSelected)
		return;
	debug("WT OnWidgetSelected nodeid:" + nodeId + " sel:" + isSelected);
	var selection = widgetTreeView.selection;
	try {
		// selection.selectEventsSuppressed only stops event temporarily.
		// all those events occur when the flood gates aer opened.
		ignoreWidgetTreeSelect = true; // otherwise we get recursion.
		treeitems = widgetTree.getElementsByTagName("treeitem");
		for (var i=0; i<treeitems.length; i++) {
			var treeitem = treeitems.item(i);
			var id = treeitem.getAttribute("xulapp-id");
			if (id == nodeId) {
				Utility.ensureParentIsOpen(widgetTree, treeitem);
				var visibleIndex = widgetTreeView.getIndexOfItem(treeitem);
				widgetTree.treeBoxObject.ensureRowIsVisible(visibleIndex);
				if (!selection.isSelected(visibleIndex)) 
					selection.rangedSelect(visibleIndex, visibleIndex, augmentSelection);
				break;
			}	
		}
	}
	finally {
		ignoreWidgetTreeSelect = false;
		if (isSelected)
			widgetTree.boxObject.invalidate();
	}
	if (this.propInspector) {
		this.propInspector.OnWidgetSelected(nodeId, isSelected)
	}
}

function OnAttributeChanged(nodeId, attributeid, oldValue, newValue, originatingWindow) {
	if (originatingWindow == window) return;
	//debug("widget:" + srcNode.localName + " Attribute:" + attributeid + " has  changed - old:" + oldValue + ",new:" + newValue);
	var treeitem = findByNodeId(nodeId);
	if (treeitem) { // only deal with changes to visible/filtered elements.
		var srcNode = xulDoc.getIdMapNode(nodeId);
		var newtreeitem = buildtreeitem(srcNode);
		treeitem.firstChild.firstChild.setAttribute("label", newtreeitem.firstChild.firstChild.getAttribute("label"));
	}
}

// WARNING: Do not use Venkman to debug during D&D! Locks up every time.
// WARNING: Latest news Venkman nolonger locks - it just doesn't allow debugging during D&D.
function canDropWidgetBeforeAfter(index, before) {
	dropItem = widgetTreeView.getItemAtIndex(index);
	dropWhere = before?DROP_BEFORE:DROP_AFTER;
	return canDropWidgetHere();
}

function canDropWidgetOn(index) {
	dropItem = widgetTreeView.getItemAtIndex(index);
	dropWhere = DROP_ON;
	return canDropWidgetHere();
}
function canDropWidgetHere() {
var dropSrcNode;
    dragSession = dragService.getCurrentSession();
    if (! dragSession)
        return false;
    if (dragSession.isDataFlavorSupported(MIMETYPE_NEW_WIDGET)) {
    	dragItem = dragSession.sourceNode;
    	dragItemName = dragItem.getAttribute("widgetid");
    	//debug("canDropWidgetHere" + dragItemName);
		dropSrcNode = getSrcNode(dropItem);    		
		if (!dropSrcNode)
			return false;
		dropItemParentName = null;
		if (dropSrcNode.nodeType == NODE_TYPE_PROCESSING_INSTRUCTION) 
			dropItemParentName = "XULDocument";
		else {	
			dropItemParentName = (dropWhere == DROP_ON)? dropSrcNode.localName: dropSrcNode.parentNode.localName;
			if (!dropItemParentName)
				return false;
		}
        return canDropHereEx(dropItemParentName, dragItemName, dropWhere, MIMETYPE_NEW_WIDGET);
    }
    if (dragSession.isDataFlavorSupported(MIMETYPE_OLD_WIDGET)) {
    	if (widgetTreeView.selection.count > 1) {
    		return false;
    	}
    	dragItem = getSelectedItems(widgetTree )[0];
		var dragSrcNode = getSrcNode(dragItem);   
		if (!dragSrcNode)
			return false;
    	dragItemName = dragSrcNode.localName;
    	//debug("dragItemName:" + dragItemName);
    	dropSrcNode = getSrcNode(dropItem);    		
		if (!dropSrcNode)
			return false;
		dropItemParentName = null;
		if (dropSrcNode.nodeType == NODE_TYPE_PROCESSING_INSTRUCTION) 
			dropItemParentName = "XULDocument";
		else {	
			dropItemParentName = (dropWhere == DROP_ON)? dropSrcNode.localName: dropSrcNode.parentNode.localName;
			if (!dropItemParentName)
				return false;
		}
		return canDropHereEx(dropItemParentName, dragItemName, dropWhere, MIMETYPE_OLD_WIDGET);
    }
    return false;
}
function canDropHereEx(dropItemParentName, dragItemName, dropWhere, flavourType) {
// WARNING: Do not use Venkman to debug in D&D! Locks up every time.
	if (dragItemName == "comment") return true; // can drop comment anywhere.
	//debug("candrophere1:" + dragItemName + " into " + dropItemParentName + " action:" + dropWhere);
	// deal with PI.s or <window> 
	if (dropItemParentName == "XULDocument") {
		// dont allow window to move or to drop anything after the window.	
		if (dragItemName == "window" )
			return false;
	}
	switch (flavourType) {
	case MIMETYPE_NEW_WIDGET:
		var widgetDef = xulapp.widgets.getElementById(dragItemName);
		if (!widgetDef) return false;
		var widgetClass = widgetDef.getAttribute("class");
		if (!widgetClass) widgetClass = "widget";
		//debug("candrophere4:" + widgetClass + ":" + dropItemParentName);
		if (widgetClass == "PI" && dropItemParentName == "XULDocument")
			return true;
		// what will we be dropping - it allowed here?
		var template = getTemplate(widgetDef, dropItemParentName);
		//debug("candrophere5:" + (template));
		if  (!template)
			return false;
		var elems = template.getElementsByTagName("*");
		if  (elems.length == 0)
			return false;
		dragItemName = elems.item(0).localName;
		//debug ("candrophere2:" + dropItemParentName + ":" + dragItemName);
		// Achtung: Drop through!
	case MIMETYPE_OLD_WIDGET:
		var parentDef = xulapp.widgets.getElementById(dropItemParentName);
		if (!parentDef)
			return false;

		acceptschildren = parentDef.getAttribute("acceptschildren");
		if (!acceptschildren)
			return false;
		//debug ("candrophere3:" + acceptschildren + ":" + dragItemName);
		acceptschildren = acceptschildren.split(/\s/);
		return acceptsLocalName(acceptschildren, dragItemName);
	default:
		debug ("canDropHere - unknown drag source.");
		return false;
	}
}

function dropWidget(index, dropWhere)
{
	//debug("Dropped" + index + dropWhere);
    dragSession = dragService.getCurrentSession();
    if (! dragSession)
        return false;
    if (dragSession.isDataFlavorSupported(MIMETYPE_NEW_WIDGET)) {
    	InsertNewWidget(dragItemName, dropItemParentName, dropWhere, dropItem);
	}
    else if (dragSession.isDataFlavorSupported(MIMETYPE_OLD_WIDGET)) {
    	InsertOldWidget(dragSession, dragItemName, dropItemParentName, dropWhere, dropItem);
	}
}


var widgetObserver = {
	
	onDragStart: function (event, transferData, action){
		//!!! This is only called for drags originitaing within the WidgetEditor tree - see Widgetbar for other source.
	    //TODO allow multiple selection - ensure selection is legal
	    if (widgetTreeView.selection.count.length > 1) {
	    	debug("multiple selection is not yet supported.");
	    	return false;
	    }
	    //debug("startdrag");
		//var treeitem = widgetTree.selectedItems[0];
		//xulapp.dropdata = treeitem;
	    transferData.data = new TransferData();
		// we don't seem to be able to pass XULElements as XPCOM objects - they won't event QueryInterface.
		// so bypass the d&d system and use a global dropdata.
	    transferData.data.addDataForFlavour(MIMETYPE_OLD_WIDGET,"see xulapp.dropdata");
	},
	
};


function acceptsLocalName(acceptschildren, localName) {
	for (var i=0; i<acceptschildren.length; i++) {
		if (acceptschildren[i] == "*box") {
			if (acceptsLocalName(xulapp.boxAcceptsChildren, localName))
				return true;
		}		
		if (acceptschildren[i] == localName)
				return true;
	}
	return false;
}

function InsertNewWidget(widgetName, dropItemParentName, dropWhere, dropItem) {
	//debug("Drop:" + widgetName + " Action:" + dropWhere + " " + dropItem.firstChild.firstChild.getAttribute("label"));
	var widgetDef = xulapp.widgets.getElementById(widgetName);
	if (!widgetDef) return;
	
	// who are we dropping on.
	var nodeId =  dropItem.getAttribute("xulapp-id");
	if (!nodeId)
		return;
	var srcNode = xulDoc.getIdMapNode(nodeId); 
	//debug("drop srcNode" + srcNode.localName);
	var srcParent = (dropWhere == DROP_ON) ? srcNode : srcNode.parentNode;
		
	var widgetClass = widgetDef.getAttribute("class");
	if (!widgetClass) widgetClass = "widget";
	var templateClone;
	switch (widgetClass) {
	case "widget":	
		// what will we be dropping?
		var template = getTemplate(widgetDef, dropItemParentName);
		//debug("template" + template);
		if (!template) return;
		
		// if empty template not droppable.
		if (!template.hasChildNodes()) return;
	
		templateClone = cloneTemplate(template); 
		templateClone = templateClone.getElementsByTagName("*").item(0);// we allow only single rooted templates.

		if (srcParent.localName == "stack" ) {
			templateClone.setAttribute("left", "0");
			templateClone.setAttribute("top", "0");		
		}
		break;
	case "PI":	
		templateClone = document.createProcessingInstruction(widgetDef.getAttribute("target"), widgetDef.getAttribute("data"));
		break;
	case "COMMENT":
		try {
			templateClone = document.createComment(widgetDef.getAttribute("data"));
		}
		catch (e) {alert(e); return; } // trap Mozilla not implemented.
		break;
	default:
		return; // XXX handle comments and DTD.
	}

	switch (dropWhere) {	
	case DROP_BEFORE:
		srcNode.parentNode.insertBefore(templateClone, srcNode);
		break;
	case DROP_AFTER:
		srcNode.parentNode.insertBefore(templateClone, srcNode.nextSibling);
		break;
	case DROP_ON:
		srcNode.appendChild(templateClone);
		break;
	default:
		throw "Unknown dropWhere:" + dropWhere;
	}
	xulDoc.addSrcNodeToIdMap(templateClone);
	//TXN Fix me when undo/redo is implemented
	xulDoc.isModified = true;
	
	fillTree(dropItem, dropWhere, templateClone)
	// Update editor if active.
	if (xulapp.currentDocument && xulapp.currentDocument.editorWindow) {
		xulapp.currentDocument.editorWindow.updateDocument(srcNode, dropWhere, templateClone);
	}
}
function InsertOldWidget(dragSession, dragItemName, dropItemParentName, dropWhere, dropItem) {
	debug("dropWhere:" + dropWhere + " " + dropItem + "dragAction:" + dragSession.dragAction);
	// who are we dropping on.
	var dropSrcNode = getSrcNode(dropItem); 
	//debug("dropSrcNode" + dropSrcNode.localName);

	//FIXME allow for multiple selection drag.
    var	dragItem = getSelectedItems(widgetTree )[0];
    // debug("dragItem" + dragItem);
	var dragSrcNode = getSrcNode(dragItem);   

	// if we are copying we need to clone the srcNode and then
	// construct a new set of treeitem(s) for it.
	if (dragSession.dragAction == DRAGACTION_COPY) {
		dragSrcNode = cloneTemplate(dragSrcNode);
		xulDoc.addSrcNodeToIdMap(dragSrcNode);
		var newChildren = document.createElement("treechildren");
		// create required items as children of newChildren
		fillTree(newChildren, DROP_ON, dragSrcNode)
		dragItem = newChildren.firstChild;
	}

	try {
		switch (dropWhere) {
		case DROP_BEFORE:
			dropSrcNode.parentNode.insertBefore(dragSrcNode, dropSrcNode);
			dropItem.parentNode.insertBefore(dragItem, dropItem);
			//TXN Fix me when undo/redo is implemented
			xulDoc.isModified = true;
			break;
		case DROP_AFTER:
			dropSrcNode.parentNode.insertBefore(dragSrcNode, dropSrcNode.nextSibling);
			dropItem.parentNode.insertBefore(dragItem, dropItem.nextSibling);
			//TXN Fix me when undo/redo is implemented
			xulDoc.isModified = true;
			break;
		case DROP_ON:
			dropSrcNode.appendChild(dragSrcNode);
			//TXN Fix me when undo/redo is implemented
			xulDoc.isModified = true;

			var treeChildren = dropItem.getElementsByTagName("treechildren");
			if (treeChildren.length == 0)
				treeChildren = null;
			else
				treeChildren = treeChildren.item(0);
			if (!treeChildren) {
				treeChildren = document.createElement("treechildren");
				treeChildren.setAttribute("flex","1");
				dropItem.appendChild(treeChildren);
			}
			treeChildren.appendChild(dragItem);
			break;
		default:
			throw "Unknown dropWhere:" + dropWhere;
		}
	} catch (e) {} // attempts to drop container within itself generate exceptions.
	
	// Update editor if active.
	if (xulapp.currentDocument && xulapp.currentDocument.editorWindow) {
		xulapp.currentDocument.editorWindow.updateDocument(dropSrcNode, dropWhere, dragSrcNode);
	}
}


function getTemplate(widgetDef, dropItemParentName) {
	var templates = widgetDef.getElementsByTagName("template");
	//debug("getTemplate:" + widgetDef.getAttribute("id") + ":" + templates.length + ":" + dropItemParentName)
	var template = null;
	for (var  i=0; i<templates.length; i++) {
		template = templates.item(i);
		var matchparent = template.getAttribute("matchparent");
		if (!matchparent || matchparent == "*") // matches all.
			break;
		if (matchparent.indexOf(dropItemParentName) != -1) 
			break; // success.
		template = null;
	}
	return template;
}

// deep clone of template - while patching ids
function cloneTemplate(template) {
	//debug("Cloned");
	var clone = template.cloneNode(false);
	if (template.nodeType == NODE_TYPE_ELEMENT)	
		patchTemplateIds(xulDoc.srcDOM, clone);
	for (var i=0; i<template.childNodes.length; i++) {
		cloneChild = cloneTemplate(template.childNodes[i]);
		clone.appendChild(cloneChild);
	}
	return clone;
}

function patchTemplateIds(srcDOM, cloneNode) {
	// srcDOM is necessary to detect duplicated ids already issued.
	var id = cloneNode.getAttribute("id");
	var widgetType = cloneNode.localName;
	do {
		if (widgetType in xulapp.nextWidgetId)
			xulapp.nextWidgetId[widgetType] += 1;
		else
			xulapp.nextWidgetId[widgetType] = 1;
		var widgetId = widgetType + xulapp.nextWidgetId[widgetType];
	} while (srcDOM.getElementById(widgetId) || getNodeElementById(cloneNode,widgetId));	
	cloneNode.setAttribute("id",widgetId);

	if (id == "*auto") {
		// Special cases to copy id into selected attributes:
		var x = cloneNode.getAttribute("label");
		if (x && x == "*id") cloneNode.setAttribute("label", widgetId);
		var x = cloneNode.getAttribute("value");
		if (x && x == "*id") cloneNode.setAttribute("value", widgetId);
	}
}

function loadTreeFromDOM(xulDoc) {
	var widgetTreeChildren = document.getElementById("widgetTree-children");
	// clear old tree data.
	while (widgetTreeChildren.hasChildNodes()) {
		widgetTreeChildren.removeChild(widgetTreeChildren.firstChild); 
	}
	if (!xulDoc) return;	
	var srcDOM = xulDoc.srcDOM;
	if (!srcDOM) return;
	var nodes = srcDOM.childNodes;
	for (var i=0; i<nodes.length; i++) {
		fillTree(widgetTreeChildren, DROP_ON, nodes.item(i));
	}	
}

// a recursive function to re/build the DOM tree
function fillTree(treeParent, action, srcNode) {
	var newtreeitem = buildtreeitem(srcNode);
	if (!newtreeitem) return null;
	switch (action) {
	case DROP_ON:
		var treeChildren;
		if (treeParent.localName == "treechildren")
			treeChildren = treeParent;
		else {	
			treeChildren = treeParent.getElementsByTagName("treechildren");
			if (treeChildren.length > 0)
				treeChildren = treeChildren.item(0);
			else {
				treeChildren = document.createElement("treechildren");
				treeParent.appendChild(treeChildren);
			}
		}
		treeChildren.appendChild(newtreeitem);
		break;
	case DROP_BEFORE:
		treeParent.parentNode.insertBefore(newtreeitem, treeParent); 
		break;
	case DROP_AFTER:	
		treeParent.parentNode.insertBefore(newtreeitem, treeParent.nextSibling);
		break;
	default:
		throw "Unknown action "  + action;
	}

	if (srcNode.hasChildNodes()) {
			var newtreechildren = document.createElement("treechildren");
			newtreechildren.setAttribute("flex","1");
			newtreeitem.appendChild(newtreechildren);			
		for (var i = 0; i < srcNode.childNodes.length; i++) {
			if (srcNode.childNodes.item(i).nodeType == NODE_TYPE_ELEMENT)
				this.fillTree(newtreechildren, DROP_ON, srcNode.childNodes.item(i));
		}
	}
	return newtreeitem;

	//========================================================================
}

function buildtreeitem(srcNode) {
	var isContainer = false;
	var treecellLabel = "???";
	var treeitem;
	var itemType;

	switch (srcNode.nodeType) {
	case NODE_TYPE_ELEMENT:
		itemType = srcNode.localName;			
		// flag as container even if currently empty.
		var widgetDef = xulapp.widgets.getElementById(srcNode.localName);
		isContainer = (widgetDef && widgetDef.hasAttribute("container") || srcNode.hasChildNodes());
		var id = srcNode.getAttribute("id");
		// build string representation of widget.
		treecellLabel =  "<" + srcNode.localName;
		if (id && id.length > 0) treecellLabel += " id='" + id + "'";
		treecellLabel += addAttrIfPresent(srcNode, "xulapp-id"); //FIXME remove after selection working.
		treecellLabel += addAttrIfPresent(srcNode, "label");
		treecellLabel += addAttrIfPresent(srcNode, "value");
		treecellLabel += addAttrIfPresent(srcNode, "src");
		treecellLabel += addAttrIfPresent(srcNode, "key");
		treecellLabel += addAttrIfPresent(srcNode, "oncommand");
		if (isContainer)		
			treecellLabel += ">";
		else	
			treecellLabel += "/>";		
		break;
	case NODE_TYPE_COMMENT:
		itemType = "comment";
		if (srcNode.data.length > MAX_LABEL_LENGTH)
			treecellLabel = srcNode.data.substr(0,MAX_LABEL_LENGTH) + "...";
		else
			treecellLabel = srcNode.data;
		treecellLabel = "<!--" + treecellLabel.replace(/[\n\r]/g,"") + "-->";
		break;
	case NODE_TYPE_DOCUMENT_TYPE:
		itemType = srcNode.target;
		treecellLabel = "<!DOCTYPE window";
			if (srcNode.systemId)
				treecellLabel += ' SYSTEM "' + srcNode.systemId + '"';
		treecellLabel += ">";		
		break;
	case NODE_TYPE_PROCESSING_INSTRUCTION:
		itemType = srcNode.target;
		treecellLabel = "<?" + srcNode.target + " " + srcNode.data + "?>";
		break;
	default:
		return;
	}
	treeitem = document.createElement("treeitem");
	setSrcNode(treeitem, srcNode);
	treeitem.setAttribute("class", "widget-item");
	treeitem.setAttribute("itemtype", itemType);			

	treerow = document.createElement("treerow");
	treecell = document.createElement("treecell");
	treecell.setAttribute("label",treecellLabel);
	treecell.setAttribute("class","treecell-indent");
	treecell.setAttribute("flex","1");
	treerow.appendChild(treecell);
	treeitem.appendChild(treerow);

	if (isContainer)
		treeitem.setAttribute("container","true");

	return treeitem;
}
function addAttrIfPresent(srcNode, attrName) {
	var theAttr = srcNode.getAttribute(attrName);
	if (theAttr) {
		// crop labels if too long.
		if (theAttr.length > MAX_LABEL_LENGTH) theAttr = theAttr.substr(0,MAX_LABEL_LENGTH) + "...";
		theAttr = " " + attrName + "='" + theAttr + "'";
	}
	else
		theAttr = "";
	return theAttr;	
}

function getSrcNode (treeitem) {
	if (!treeitem)
		return null;
	if (!treeitem.getAttribute)
		return null;
	var id = treeitem.getAttribute("xulapp-id");
	if (!id)
		return null;
	var srcNode = xulDoc.getIdMapNode(id);
	return srcNode;
}
function setSrcNode(treeitem, srcNode) {
	var id = xulDoc.getIdMapId(srcNode);
	treeitem.setAttribute("xulapp-id", id);
}

function findByNodeId(nodeId) {
	treeitems = widgetTree.getElementsByTagName("treeitem");
	for (var i=0; i<treeitems.length; i++) {
		var treeitem = treeitems.item(i);
		var id = treeitem.getAttribute("xulapp-id");
		if (id == nodeId) {
			//debug("FOUND:" + treeitem);
			return treeitem;
		}	
	}
	return null;
}

function onDeleteSelection(event) {
	var selectedItems = getSelectedItems(widgetTree);
	for (var i=0; i<selectedItems.length; i++) {
		var treeitem = selectedItems[i];
		var id = treeitem.getAttribute("xulapp-id");
		var srcNode = xulDoc.getIdMapNode(id);

		if (srcNode.isSameNode(xulDoc.srcDOM.documentElement)) { // can't delete root element.
			continue;
		}
		if (!srcNode) continue;
		if (srcNode.nodeType == srcNode.NODE_TYPE_ELEMENT) {
			var widgetDef = xulapp.widgets.getElementById(srcNode.localName);
			if (!widgetDef) continue;
			var templates = widgetDef.getElementsByTagName("template");
			if (templates.length <= 0 ) continue; // If we can't add it - we can't delete it.
		}	
		// OK Take a deep breath - It seems to be OK to select a parent AND some of its children for deletion at the same time.
		// The selectedItems list must be dynamically pruned - way to go!
		treeitem.parentNode.removeChild(treeitem);
		srcNode.parentNode.removeChild(srcNode);
		//TXN Fix me when undo/redo is implemented
		xulDoc.isModified = true;

		if (xulDoc.editorWindow) {
			xulDoc.editorWindow.updateDocument(null, "delete", srcNode);
		}
	}
}

function toOpenWindowByType( inType, uri )
{
	var windowManager = Components.classes['@mozilla.org/rdf/datasource;1?name=window-mediator'].getService();

	var	windowManagerInterface = windowManager.QueryInterface( Components.interfaces.nsIWindowMediator);

	var topWindow = windowManagerInterface.getMostRecentWindow( inType );
	
	if ( topWindow )
		topWindow.focus();
	else
		window.open(uri, "_blank", "chrome,menubar,toolbar,resizable,scrollbars");
}


function OnUnload(event) {
	if (xulDoc) { // Unregister our listeners.
		xulDoc.unregisterSelectionListener(OnWidgetSelected);
		xulDoc.unregisterAttributeListener(OnAttributeChanged);	
	}
	if (this.widgetbar)
		this.widgetbar.close();
	if (this.propInspector)
		this.propInspector.close();
	xulapp.widgetTreeEditor = null;
}


