/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
 
const NODE_TYPE_ELEMENT = 1;
const NODE_TYPE_ATTRIBUTE = 2;
const NODE_TYPE_TEXT = 3;
const NODE_TYPE_CDATA = 4;
const NODE_TYPE_ENTITY_REF = 5;
const NODE_TYPE_ENTITY = 6;
const NODE_TYPE_PROCESSING_INSTRUCTION  = 7; 
const NODE_TYPE_COMMENT  = 8;
const NODE_TYPE_DOCUMENT  = 9;
const NODE_TYPE_DOCUMENT_TYPE  = 10;
const NODE_TYPE_DOCUMENT_FRAGMENT  = 11;
const NODE_TYPE_NOTATION  = 12;
 
function assert(test, msg) {
	if (!test) {
		alert("Assert failed:\n" + msg + "\n" + getStackTrace());
		throw msg;
	}
}
/** Write debug string to javascript console.
**/
function debug(aText) {
  var csClass = Components.classes['@mozilla.org/consoleservice;1'];
  var cs = csClass.getService(Components.interfaces.nsIConsoleService);
  cs.logStringMessage(aText);
}

function getStackTrace() {
  var frame = Components.stack.caller;
  var str = "";
  while (frame) {
    if (frame.filename)
      str += frame.filename + ", Line " + frame.lineNumber;
    else
      str += "[No file name]";
    
    str += " --> ";
    
    if (frame.functionName)
      str += frame.functionName;
    else
      str += "[No function name]";
      
    str += "\n";
    
    frame = frame.caller;
  }
  
  return str;
}

function getElementById(id) {

	var nodelist = this.getElementsByTagName("*");
	for (var i=0; i < nodelist.length; i++) {
		var idAttr = nodelist[i].getAttribute("id");
		if (!idAttr) continue;
		if (idAttr == id) {
			return nodelist[i];
		}	
	}
	return null;
}

function getNodeElementById(node, id) {

	var nodelist = node.getElementsByTagName("*");
	for (var i=0; i < nodelist.length; i++) {
		var idAttr = nodelist[i].getAttribute("id");
		if (!idAttr) continue;
		if (idAttr == id) {
			return nodelist[i];
		}	
	}
	return null;
}

function isWindows() {
	return navigator.appVersion.indexOf("Win") != -1; 
}

function isUNIX() {
	return  ( navigator.appVersion.indexOf("X11") != -1 || navigator.appVersion.indexOf("nux") ) != -1;
}

function isMacfunction() {
 return !this.IsWindows() && !this.IsUNIX;
}


function getNewline() {
	if (isWindows())
		return "\n\r";
	else	
		if (isUNIX())
			return "\n";
		else
			if (isMac())
				return "\r";
			else	
				return "\n"; // default
}

function getDirSeparator() {
	if (isWindows())
		return "\\";
	else	
		return "/";
}
// For use in debugger to list interfaces an object supports
function listInterfaces(anObject) {
	var result = "";
	for (var anInterface in Components.interfaces) {
		try {
			anObject.QueryInterface(Components.interfaces[anInterface]);
			result += anInterface + "  ";
		}
		catch (e) {}
	}
	return result;
}


function UtilityClass() {
	this.NEWLINE = getNewline();
	this.DIR_SEPARATOR = getDirSeparator();
}


/**
 * 	Get a component.
 *  @param aURI The URi  of the component. E.g. "@mozilla.org/xmlextras/domparser;1"
 *  @param interface required. E.g.nsIDOMParser
 *	@return the service.
**/
UtilityClass.prototype.getComponent = function(aURI, aInterface) {
    try {
		return Components.classes[aURI]
						.createInstance()
						.QueryInterface(Components.interfaces[aInterface]);
    } catch (ex) {
    	throw "Error getting component: " + aURI + ", " + aInterface + "\n" + ex;
    }
    
}
/**
	load a DOM document asynchronously from a URI.
	@param uri the uri to open.
	@param namsepaceURI the namespace uri of the created document.
	@param onloadCallback a function to call when the document is loaded.
	@return the DOM document.
**/
UtilityClass.prototype.asyncLoadDOMFromURI = function(loadMimeType, uri, namespaceURI, onloadCallback) {
    var xmlDoc = document.implementation.createDocument(namespaceURI, "", null); 
    xmlDoc.addEventListener("load", onloadCallback, false); 
    xmlDoc.addEventListener("abort", onloadCallback, false); 
    xmlDoc.addEventListener("error", onloadCallback, false); 

    xmlDoc.load(uri, loadMimeType);
	 if (Components.lastResult != Components.results.NS_OK)  {
	 	alert(Components.lastResult);
	 }
	return xmlDoc;
}
/**
	load a DOM document from a URI.
	@param uri the uri to open.
	@return the DOM document.
**/
UtilityClass.prototype.loadDOMFromURI = function(loadMimeType, uri) {
	var dom = null;
	try {
		var xml = this.loadSrcFromURI(uri);
		var domParser = this.getComponent("@mozilla.org/xmlextras/domparser;1", "nsIDOMParser");
		dom =  domParser.parseFromString(xml, loadMimeType)
		//debug(dom);
		if (Components.lastResult != Components.results.NS_OK) 
			throw "XML parseFromString failed internally for " + uri; // internal failure.
		else {
			var documentElement = dom.documentElement;
			if (documentElement && documentElement.localName == "PARSERERROR") {
				var s = new XMLSerializer();
				var str = s.serializeToString(dom);
				alert(uri + Utility.NEWLINE + str);
				throw "XML parse error in " + uri;
			}
		}		
		// patch up missing getElementById method.
		// XULElements are defective as well use getNodeElementById
		dom.getElementById = getElementById;
	}
	catch (e) { alert(e); }
	return dom; 
}

/**
	load a source file document from a URI.
	@param uri the uri to open.
	@return the src string.
**/
UtilityClass.prototype.loadSrcFromURI = function(uri) {
	var theFile = new File(uri);
	theFile.open();
	var src = theFile.read();
	theFile.close();
	return src;
}

/** 
 * Display object properties
 * @param the object
**/
UtilityClass.prototype.dumpObject = function(objName, obj) {
	debug(objName);
	for (var i in obj) {
		try {
			debug(i + "=" + obj[i]);
		}
		catch (e) {debug(i + e); }
	}
}

UtilityClass.prototype.dumpElement = function(node) {
	if (node == undefined || node == null) {
		alert(node);
		return;
	}	
	var msg = "Tag:" + node.tagName + "\n";
	for (var i=0; i < node.attributes.length; i++) {
		msg += node.attributes[i].name + "=" + node.attributes[i].value + "\n";
	}
	debug(msg);
}

/** Remove an element from an array and closeup following entries.
 * If theEntry occurs more than once only the first occurance is removed.
 * If the entry is not found do nothing.
**/
UtilityClass.prototype.removeArrayEntry = function(theArray, theEntry) {
	for (var i=0; i<theArray.length; i++) {
		if (theArray[i] === theEntry) {
			return theArray.slice(i);
		}
	}
	return theArray;
}

/** Open parent containers for selected treeitem.
 *ensureElementIsVisible should do this - but until then...:
**/
UtilityClass.prototype.ensureParentIsOpen = function(tree, treeItem) {
	if (treeItem.localName != "treeitem") return;
	var element = treeItem.parentNode;
	while (element !== tree) {
		if (element.localName == "treeitem") {
			var isOpen = element.getAttribute("open");
			if (!isOpen || isOpen != "true") 
				element.setAttribute("open","true");
		}	
		element = element.parentNode;
		if (!element) break; // failsafe.
	}
}
UtilityClass.prototype.ensureElementIsVisible = function(tree, treeitem) {
	treeitems = widgetTree.getElementsByTagName("treeitem");
	for (var i=0; i<treeitems.length; i++) {
		if (treeitem == treeitems.item(i)) {
			tree.treeBoxObject.ensureRowIsVisible(i);
			return;
		}
	}

}

/** document.doctype appears to return null for documents loaded with DOMPArser.**/
UtilityClass.prototype.getDocType = function(document) {
	for (var i=0; i<document.childNodes.length; i++) {
		if (document.childNodes.item(i).nodeType == NODE_TYPE_DOCUMENT_TYPE)
			return document.childNodes.item(i);
	}
	return null;
}

UtilityClass.prototype.xmlEncode = function(val) {
	val = val.replace(/[&]/g,"&amp;");
	val = val.replace(/</g,"&lt;");
	val = val.replace(/>/g,"&gt;");
	val = val.replace(/\"/g,"&quot;"); //"fool my text editor syntax checker into thinking the open string has ended.
	return val;
}
UtilityClass.prototype.viewDOM = function(xmlDoc, theTitle) {
	window.openDialog("chrome://xulapp/content/utility/viewDOM/viewDOM.xul", "XML Viewer " + theTitle, "resizable=yes,chrome,titlebar",xmlDoc, theTitle);				
}

var Utility = new UtilityClass();


function NullEnumerator() {
}
NullEnumerator.prototype.hasMoreElements = function() {
	return false;
}
NullEnumerator.prototype.getNext = function() {
	return null;
}

function ArrayEnumerator(itemsArray, clone) {
	if (clone)
		this.itemsArray = itemsArray.concat([]); // shallow copy.
	else	
		this.itemsArray = itemsArray;
	this.index = 0;
}
ArrayEnumerator.prototype.hasMoreElements = function() {
	return (this.index < this.itemsArray.length);
}
ArrayEnumerator.prototype.getNext = function() {
	if (this.index < this.itemsArray.length)
		return this.itemsArray[this.index++];
	else	
		return null;
}
