/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
 
/**
	This hash map implementation piggy-back's on the javascript object properties
	hash map (which is ten times faster than anything I could write in javascript).
	The only snag is the pre-defined properties of Object. We dodge these by prepending
	a $ infront of our names which collide. This prevents the creation of new strings for the majority case.
**/
function HashMap() {
	this.anObject = {};
	this.map = {};
	this.length = 0;
}
HashMap.prototype.put = function(key, value) {
	if (key in this.anObject)
		this.map["$" + key] = value;
	else
		this.map[key] = value;
	this.length++;	
}
HashMap.prototype.get = function(key) {
	if (key in this.anObject)
		return this.map["$" + key];
	else
		return this.map[key];
}
HashMap.prototype.remove = function(key) {
	if (key in this.anObject) {
		if ("$" + key in this.map) {
			delete this.map["$" + key];
			this.length--;
		}	
	}		
	else {
		if (key in this.map) {
			delete this.map[key];
			this.length--;
		}	
	}
}

HashMap.prototype.exists = function(key) {
	if (key in this.anObject) 
		return "$" + key in this.map;
	else
		return key in this.map;
}
HashMap.prototype.keys = function() {
	var k = [];
	for (var key in this.map) {
		if (key in this.anObject)
			continue;
		if (key.substr(0,1) == "$") 
			k[k.length] = key.substr(1);
		else
			k[k.length] = key;
	}
	return k;
}


