/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
 
/**
	This Transaction manager implements the same interface as nsITransactionManager
	(but not fully - it is intended as a temporary fix unti the real one is fixed).
**/
function TLink(aTransaction, batch) {
	this.batch = batch;
	this.next = null;
	this.prev = null;
	this.aTransaction = aTransaction;
}
function TransactionManager() {
	// items in a batch are tagged with the same batch no.
	this.inBatch = false; 
	this.currentBatch = 0;
	this.maxTranCount = -1;
	// the undo stack.
	this.stackBottom = null;
	this.stackTop = null;
	this.undoCount = 0; // individual transactions are counted even in batches.
	this.redoCount = 0;
}
TransactionManager.prototype.__defineSetter__( "maxTransactionCount", function(maxCount) {
	this.maxTranCount = Math.max(maxCount, -1);
	if (this.maxTranCount != -1) 
		this.pruneStack();
})
TransactionManager.prototype.__defineGetter__( "maxTransactionCount", function() {
	return this.maxTranCount;
})
TransactionManager.prototype.pruneStack = function() {
	while (this.undoCount > this.maxTranCount) {
		this.stackBottom = this.stackBottom.next;
		if (this.stackBottom) {
			this.stackBottom.prev.next = null;
			this.stackBottom.prev = null;
		}			
		this.undoCount--;
	}
}
TransactionManager.prototype.__defineGetter__( "numberOfUndoItems", function() {
	return this.undoCount;
})
TransactionManager.prototype.__defineGetter__( "numberOfRedoItems", function() {
	return this.redoCount;
})

TransactionManager.prototype.doTransaction = function(aTransaction) {
	// If transaction is merged then nothing else to do.
	if (this.undoCount && aTransaction.merge(this.stackBottom) )
		return;
	aTransaction.doTransaction();
	if (aTransaction.isTransient)
		return;
	var tLink = new TLink(aTransaction, ((this.inBatch)?this.currentBatch:0));
	if (!this.stackTop) {
		this.stackTop = tLink;
		this.stackBottom = tLink;
	}	
	else {
		// clear any redo transactions following stackTop.
		if (this.stackTop.next) {
			this.unlinkAll(this.StackTop);
		}	
		tLink.prev = this.stackTop;
		this.stackTop.next = tLink;
		this.stackTop = tLink;
	}
	this.undoCount++;
	this.redoCount = 0;
	// If fixed max transaction count then prune last item.
	if (this.maxTranCount != -1) 
		this.pruneStack();
}
// Unlink all tLinks following the given link (assist GC).
TransactionManager.prototype.unlinkAll = function(tLink) {
	if (tLink) {
		while (tLink.next) {
			tLink = tLink.next;
			tLink.prev.next = null;
			tLink.prev = null;
		}
	}	
}

TransactionManager.prototype.undoTransaction = function() {
	while (this.undoCount > 0) {
		this.stackTop.aTransaction.undoTransaction();
		var batch = this.stackTop.batch;
		this.stackTop = this.stackTop.prev;
		this.undoCount--;
		this.redoCount++;
		if (batch == 0) // Not batched - out of here
			break;
		if (this.undoCount <= 0 || this.stackTop.batch != batch) // Same batch ?
			break;
	}
}
TransactionManager.prototype.redoTransaction = function() {
	while (this.redoCount > 0) {
		if (!this.stackTop)
			this.stackTop = this.stackBottom;
		else
			this.stackTop = this.stackTop.next;
		this.stackTop.aTransaction.redoTransaction();
		var batch = this.stackTop.batch;
		this.undoCount++;
		this.redoCount--;
		if (this.stackTop.batch == 0)
			break;
		if (this.redoCount <= 0 || this.stackTop.next.batch != batch) // Same batch ?
			break;
	}
}

TransactionManager.prototype.clear = function() {
	this.unlinkAll(this.stackBottom);
	this.stackBottom = null;
	this.stackTop = null;
	this.undoCount = 0;
	this.redoCount = 0;
}
TransactionManager.prototype.beginBatch = function() {
	this.currentBatch++;
	this.inBatch = true;
}
TransactionManager.prototype.endBatch = function() {
	this.inBatch = false;
}
TransactionManager.prototype.peekUndoStack = function() {
	if (this.stackTop)
		return this.stackTop.aTransaction;
	else	
		return null;
}
TransactionManager.prototype.peekRedoStack = function() {
	if (this.redoCount > 0) {
		if (!this.stackTop)
			return this.stackBottom;
		else	
			return this.stackTop.next.aTransaction;
	}
	else
		return null;
}
TransactionManager.prototype.getUndoList = function() {
	assert(false,"not implemented");
}
TransactionManager.prototype.getRedoList = function() {
	assert(false,"not implemented");
}
TransactionManager.prototype.AddListener = function(aListener) {
	assert(false,"not implemented");
}
TransactionManager.prototype.RemoveListener = function(aListener) {
	assert(false,"not implemented");
}
